package visl;

import java.applet.Applet;
import java.awt.Button;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;

import visl.rasmus.resources.Images;
import visl.rasmus.resources.Sounds;
import visl.rasmus.utils.Loader;
import visl.wordfall.gui.MainFrame;
import visl.wordfall.resources.Miscellaneous;
import visl.resources.Dict;
import visl.resources.ShortToLong;
import visl.utils.ClassLoader;

/** This is the frontend of the "WordFall" game.
 * The "body" of the game is contained in the <CODE>gui.MainFrame</CODE>. 
 * The heart of the applet is the "tick" method in MainFrame which is called 
 * from a timer thread periodically. All user interference is handled in the 
 * <CODE>handleEvent</CODE> method in the MainFrame.
 * @author Rasmus Lock Larsen, VISL, Februar 2001
 */
public class WordFall extends Applet {
	// The package that the classes are placed in.
	private Button okb = null;
	private Frame mainFrame = null;
	private static final String packagebase = "visl.";
	private static final String divbase = "rasmus.";
	// The classes to be loaded before the program can run.
	private static final String[] initialclasses =
		{
			packagebase + "wordfall.resources.Colors",
			packagebase + "wordfall.resources.Miscellaneous",
			packagebase + "wordfall.gui.MainFrame",
			packagebase + "wordfall.gui.FormBox",
			packagebase + "wordfall.gui.FallWord",
			packagebase + "wordfall.gui.CustomDialog",
			packagebase + "resources.Dict",
			//packagebase + "resources.ShortToLong",
			packagebase + divbase + "resources.Images",
			packagebase + divbase + "resources.Sounds",
			packagebase + divbase + "utils.Timer",
			packagebase + divbase + "utils.TimerListener",
			packagebase + divbase + "handler.GraphicsHandler",
			packagebase + divbase + "handler.CursorHandler",
			packagebase + divbase + "handler.DropElement",
			packagebase + divbase + "handler.DropElementHandler",
			packagebase + divbase + "handler.TextScrollComponent",
			packagebase + divbase + "handler.TextScrollHandler" };
	private static final String[] secondaryclasses = {
	};
	private boolean initOK = false;
	private String language;

	/**
	 * Initialize the applet.
	 * Sets the language, and starts the loading of classes in a seperate
	 * thread.
	 */
	public void init() {
		System.out.println("Initialising applet...");
		//
		// Get the language and location of symbolsets
		//
		// get the language
		language = getParameter("sprog");
		String displaylanguage = getParameter("displaysprog");
		// get the display language
		String dictbase = getParameter("trans");
		String imagebase = getParameter("images");
		String soundbase = getParameter("sound");
		String helpBase = getParameter("helpbase");
		String abbrvTrans = getParameter("abbrvTrans");

		Sounds.setApplet(this, soundbase);
		try {
			Sounds.localize(soundbase + "wordfall.lst");
		} catch (Exception x) {
			System.out.println("Error while initializing sounds: " + x);
		}

		//
		// Load preferences.
		// Only default preferences are available now.
		//
		Miscellaneous.localize(helpBase);
		//
		// Get the translated text strings from the server.
		//
		try {
			if (displaylanguage == null) { // use default
				Dict.localize(dictbase + "dict." + language);
				System.out.println("Using " + dictbase + "dict." + language);
			} else { // use specified display language
				Dict.localize(dictbase + "dict." + displaylanguage);
				System.out.println("Using " + dictbase + "dict." + displaylanguage);
			}
		} catch (Throwable t) {
			System.out.println(
				"Using default dictionary\nNo "
					+ dictbase
					+ "dict."
					+ language
					+ " or "
					+ dictbase
					+ "dict."
					+ displaylanguage);
			Dict.localize(); // Use default translation (English)
		}
		if (abbrvTrans != null) {
			try {
				ShortToLong.localizeArabic(
					abbrvTrans + "abbrv.long." + language,
					abbrvTrans + "abbrv.en." + language,
					abbrvTrans + "abbrv." + language);
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}
		} else {
			ShortToLong.localize();
		}

		Images.setApplet(this, imagebase);
		//
		// Load the symbol sets
		//
		/*
		Comments.setApplet(this, commentbase);
		try{
		    if (displaylanguage == null) {
			// use default display language
			Comments.localize(commentbase + "comments." + language,
					  commentbase + "sounds." + language);
		    } else { // use specified display language
			Comments.localize(commentbase + "comments." + language + "_" +
					  displaylanguage,
					  commentbase + "sounds." + language + "_" +
					  displaylanguage);
		    }
		} catch(Throwable t){
		    t.printStackTrace();
		    throw new RuntimeException("Couldn't read symbol set");
		}
		*/
		initOK = true;
	}

	/**
	 * Start the tree visualizer.
	 * This method is called by the browser after init().
	 */
	public void start() {
		if (initOK) {
			removeAll();
			add(new Label(Dict.get("Loading applet. Please wait...")));
			validate();

			// Load initially needed classes.
			ClassLoader loader = new ClassLoader(initialclasses);
			loader.run();

			removeAll();
			add(new Label(Dict.get("Starting applet. Please wait...")));
			validate();

			// Initialize a VISLFrame.
			try {
				mainFrame = new MainFrame(this, language);
				mainFrame.show();
			} catch (Throwable e) {
				// Display an error message and stop the applet.
				quit(Dict.get("Applet stopped because of an error:"), e);
				return;
			}
			// load secondary classes.
			try {
				Loader.load(secondaryclasses);
			}
			// Must catch exceptions from load
			catch (Throwable e) { // ignore errors now.
				return;
			}
		}
	}
	/**
	 * Determins whether the language needs java 1.3 and if the vm is 1.3 or newer.
	 * @param lang the game language
	 * @return true if the language does not requere a java 1.3 virtual mashine and if it does true if the vm is 1.3 or newer.
	 */
	public boolean isJava1_3Compatible(String lang) {
		if (!ShortToLong.isJava1_3Languages(lang)) {
			return true;
		}
		String v = System.getProperty("java.version");
		System.out.println("java.version = " + v);
		if (v.compareTo("1.3") < 0) {
			return false;
		}
		return true;
	}
	
	public boolean handleEvent(Event e) {
		if (e.target == okb && e.id == Event.ACTION_EVENT) {
			mainFrame.show();
			removeAll();
			return true;
		} else if (e.target instanceof Button && e.id == Event.ACTION_EVENT) {
			quit("Good-bye, Please try WordFall from a different machine!", null);
		}
		return false;
	}
	public void quit(String s, Throwable e) {
		removeAll();
		add(new Label(s));
		if (e != null) {
			add(new Label(e.toString()));
			e.printStackTrace();
		}
		validate();
		stop();
		destroy();
	}
}
