package visl.rasmus.gui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

public class Graph extends Canvas {
    public static final int BAR = 0, XY = 1;
    int x[], y[];
    String title, legend[];
    int maxy, maxly, maxll, maxlx;
    public Graph(String title, int x[], int y[], String legend[]) {
	this.title = title;
	setXY(x,y);
	setLegend(legend);
    }
    public Graph(String title, int x[], int y[]) {
	this.title = title;
	legend = null;
	maxll = 0;
	setXY(x,y);
    }
    public void paint(Graphics g) {
	Dimension d = size();
	Color col = g.getColor();
	g.setColor(Color.white);
	g.fillRect(0,0, d.width, d.height);
	Font fnt = getFont();
	FontMetrics fm = getFontMetrics(fnt);
	int fH = (fm == null ? 20 : fm.getHeight());
	int tW = (fm == null ? 8*title.length() : fm.stringWidth(title));
	int ty = 0;
	int stepmy = (maxy >= 10 ? maxy/10 : 1);
	// Find the size of the y&x-labels.
	if ((maxly == 0 || maxlx == 0) && fm != null) {
	    for (int i = 0; i < maxy; i+=stepmy) {
		ty = fm.stringWidth(""+i);
		if (ty > maxly) maxly = ty;
	    }
	    maxly += 10;
	}
	if (maxlx == 0 && fm != null) {
	    ty = String.valueOf(x[0]).length();
	    if (ty > maxlx) maxlx = ty;
	    ty = String.valueOf(x[x.length-1]).length();
	    if (ty > maxlx) maxlx = ty;
	    maxlx += 1;
	    maxlx = maxlx * fH;
	}
	// Find the size of the legends.
	if (legend != null && maxll == 0 && fm != null) {
	    for (int i = 0; i < x.length; i++) {
		ty = fm.stringWidth(""+x[i]+": " + legend[i]);
		if (ty > maxll) maxll = ty;
	    }
	    maxll += 10;
	}
	g.setColor(Color.red);
	g.drawString(title, (d.width-tW)/2, fH*2);
	g.setColor(Color.gray);
	int aW = d.width - maxly;
	if (legend != null) aW -= maxll;
	int aH = d.height - fH*3 - maxlx;
	g.drawLine(maxly, fH*2, maxly, d.height - maxlx);
	g.drawLine(maxly, d.height - maxlx, d.width, d.height - maxlx);
	float stepx = (float)aW/x.length;
	float stepy = (float)aH/maxy;
	// Draw legends
	if (legend != null) {
	    for (int i = 0; i < x.length; i++) {
		g.drawString(""+x[i]+": " + legend[i], d.width - maxll + 5, 
			      fH*(2+i));
	    }
	}
	// Draw axis and y-labels
	for (int i = 0; i < maxy; i+=stepmy) {
	    ty = d.height - maxlx - (int)(stepy*i);
	    g.drawString(""+i, 5, ty);
	    g.drawLine(maxly-5, ty, d.width-maxll, ty);
	}
	// Draw the bars
	for (int i = 0; i < x.length; i++) {
	    g.setColor(Color.gray);
	    String s = String.valueOf(x[i]);
	    for (int j = 0; j < s.length(); j++) {
		g.drawString(""+s.charAt(j), maxly + (int)(stepx*i), 
			     d.height - maxlx + (j+2)*fH);
	    }
	    g.setColor(Color.blue);
	    g.fillRect(maxly + (int)(stepx*i), 
		       d.height - maxlx - (int)(y[i]*stepy), 
		       (stepx > 1 ? (int)(stepx/2) : 1), (int)(y[i]*stepy));
	    g.drawString(""+y[i], maxly + (int)(stepx*i), 
			 d.height - maxlx - (int)(y[i]*stepy) - fH/2);
	}
	g.setColor(col);
    }
    public void setTitle(String title) { this.title = title; }
    public void setXY(int x[], int y[]) {
	this.x = x;
	this.y = y;
	maxy = 0;
	maxly = 0;
	maxlx = 0;
	for (int i = 0; i < y.length; i++)
	    if (y[i] > maxy) maxy = y[i];
    }
    public void setLegend(String sa[]) {
	if (sa.length != x.length) throw new RuntimeException("Length of legend must match length of x-values");
	maxll = 0;
	legend = new String[sa.length];
	System.arraycopy(sa, 0, legend, 0, sa.length);
    }
}




