package visl.rasmus.gui;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;

import visl.rasmus.handler.GraphicsHandler;
import visl.rasmus.resources.Images;

public class ImageCanvas extends Canvas implements GraphicsHandler {
    // *** STATIC
    public static final int 
	CENTERED = 0,
	TILED = 1,
	NORMAL = 2;
    // *** NON-STATIC
    protected Image image;
    protected Dimension d;
    protected int style, x, y;

    public ImageCanvas() {
	this((Image)null, CENTERED);
    }
    public ImageCanvas(Image image) {
	this(image, CENTERED);
    }
    public ImageCanvas(String imagefilename) {
	this(Images.getImage(imagefilename), CENTERED);
    }
    public ImageCanvas(String imagefilename, int style) {
	this(Images.getImage(imagefilename), style);
    }
    public ImageCanvas(Image image, int style) {
	this.image = image;
	d = null;
	if (image != null) {
	    int w = image.getWidth(this);
	    int h = image.getHeight(this);
	    resize(w, h);
	}
	this.style = style;
	x = 0;
	y = 0;
    }
    public void paint(Graphics g) {
	if (image != null) {
	    int w = image.getWidth(this),
		h = image.getHeight(this);
	    Dimension d = size();
	    switch (style) {
	    case CENTERED:
		g.drawImage(image, x+(d.width-w)/2, y+(d.height-h)/2, this);
		break;
	    case TILED:
		int tx = 0,ty = 0;
		while (ty < d.height) {
		    g.drawImage(image, x + tx, y + ty, this);
		    tx += w;
		    if (tx >= d.width) {
			tx = 0;
			ty += h;
		    }
		}
		break;
	    }
	}
    }
    public void resize(Dimension d) {
	this.d = d;
	super.resize(d);
    }
    public void resize(int w, int h) {
	d = new Dimension(w,h);
	super.resize(w,h);
    }
    public Dimension preferredSize() {
	if (d != null) {
	    return d;
	}
	return new Dimension();
    }
    public void updateGraphics(Graphics g, Object arg) {
	paint(g);
    }
    public void setPosition(int x, int y) {
	this.x = x;
	this.y = y;
    }
    /** Sets the image of this canvas, returns the previous image */
    public Image setImage(Image img) {
	Image im = image;
	image = img;
	repaint();
	return im;
    }
    public Image getImage() { return image; }
}




