package visl.rasmus.handler;

import java.awt.Graphics;
import java.util.Enumeration;
import java.util.Vector;

import visl.rasmus.utils.TimerListener;

/** This class can handle multiple components, as long as they inherits from
 * MultiComponent.
 * @author Rasmus Lock Larsen, July 2001 */
public final class MultiHandler implements TimerListener {
    protected Vector components;
    protected long last_time, time;
    protected boolean paused;
    public MultiHandler() {
	paused = false;
	components = new Vector();
	last_time = System.currentTimeMillis();
    }
    public void pause() { paused = true; }
    public void start() { 
	last_time = System.currentTimeMillis();
	paused = false;
    }
    public synchronized void add(MultiComponent mc) {
	components.addElement(mc);
    }
    public synchronized void remove(MultiComponent mc) {
	components.removeElement(mc);
    }
    public synchronized int size() { return components.size(); }
    public synchronized Enumeration elements() { return components.elements();}
    /** Updates the multicomponents based on their speed and acceleration */
    public synchronized void tick() {
	time = System.currentTimeMillis();
	if (!paused) {
	    for (Enumeration en = components.elements(); 
		 en.hasMoreElements();) {
		MultiComponent mc = (MultiComponent)en.nextElement();
		if (mc.isMoving()) {
		    mc.vx += mc.ax * (time - last_time);
		    mc.vy += mc.ay * (time - last_time);
		    mc.fx += mc.vx * (time - last_time);
		    mc.fy += mc.vy * (time - last_time);
		    mc.x = (int)mc.fx;
		    mc.y = (int)mc.fy;
		}
	    }
	}
	last_time = time;
    }
    public synchronized void updateGraphics(Graphics g, Object o) {
	for (Enumeration en = components.elements(); en.hasMoreElements(); ) {
	    MultiComponent mc = (MultiComponent)en.nextElement();
	    mc.paint(g);
	}
    }
    public synchronized void clear() {
	components.removeAllElements();
    }
}


