package visl.rasmus.hitserver.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;

import visl.rasmus.hitserver.data.HighScoreItem;
import visl.rasmus.hitserver.data.Sentence;
import visl.rasmus.hitserver.data.TreeNode;
import visl.rasmus.hitserver.server.HitServerInterface;
import visl.resources.Dict;

/** This class implements the public interface to the HitServer 
 * @see HitXtraClient
 * @author Rasmus Lock Larsen, VISL March 2001 
 * updated by Asad Ali Main for Encoding, April, 2007
 * */
public class HitClientPHP implements HitServerInterface {
	public static final Hashtable short2long = new Hashtable();
	protected String scriptname;
	protected BufferedReader in;
	protected String language, game, encoding, pattern;
	protected boolean ioerr;

	public static final String T10D = "T10D",
		T10W = "T10W",
		T10M = "T10M",
		T10Y = "T10Y",
		T100W = "T10D",
		T100M = "T10D",
		T100Y = "T10D",
		TALL = "TALL";

	public HitClientPHP(String scriptname, String language, String game) {
		//encoding = "ISO8859_1";
		
		 
		
		System.out.println("Changing Encoding...23/04.....1");
		
		encoding = null;
		
		ioerr = false;
		this.scriptname = scriptname;
		this.language = language;
		this.game = game;
		this.pattern = null;
		if (!Dict.isInitialized())
			Dict.localize();
		// This should be removed - not suitable to have statically linked
		// "dictionaries".
		short2long.put(T100Y,  Dict.get("This year's top 100 scores!"));
		short2long.put(T100M, Dict.get("This month's top 100 scores!"));
		short2long.put(T10W, Dict.get("This week's top 10 scores!"));
		short2long.put(
			T10D,
			Dict.get("The top 10 scores in the last 24 hours!"));
		short2long.put("en", "English");
		short2long.put("bs", "Bosnian");
		short2long.put("el", "Greek");
		short2long.put("it", "Italian");
		short2long.put("la", "Latin");
		short2long.put("pt", "Portuguese");
		short2long.put("da", "Danish");
		short2long.put("fr", "French");
		short2long.put("de", "German");

		// This should be placed elsewhere
		short2long.put("n", Dict.get("nouns"));
		short2long.put("v", Dict.get("verbs"));
		short2long.put("adj", Dict.get("adjectives"));
		short2long.put("adv", Dict.get("adverbs"));
		short2long.put("art", Dict.get("articles"));
		short2long.put("pron", Dict.get("pronouns"));
		short2long.put("prp", Dict.get("prepositions"));
		short2long.put("conj", Dict.get("conjunctions"));
		short2long.put("infm", Dict.get("infinitive markers"));
		short2long.put("num", Dict.get("numerals"));
		short2long.put("intj", Dict.get("interjections"));
	
				
		encoding ="UTF-8"; // CHANGED getEncoding(); by Asad 
		System.out.println(
			"java.version\t" + System.getProperty("java.version"));
		System.out.println("os.name     \t" + System.getProperty("os.name"));
		System.out.println("Encoding is " + encoding);
	}
	// ***********************************************************************
	// * HitServer calls
	// ***********************************************************************
	/** Reads a line from the connection, and resets the timing */
	protected String readEnc() throws IOException {
		String res = read();
		System.out.println("*****Testing Unicode String***** : " + res);
		return newString(res);
	}
	protected String read() throws IOException {
		String res = in.readLine();
		//System.out.println("-=[" + res + "]=-");
		return res;
	}
	public void connect(String args) throws IOException {
		//System.out.println("connect("+args+"), ioerr = " + ioerr + ", encoding = "+encoding);
		try {
			// Append pattern if it is set
			if (pattern != null) {
				args = args + "&pattern=" + pattern;
			}
			// Filter args
			StringBuffer nargs = new StringBuffer();
			for (int i = 0; i < args.length(); i++) {
				char ch = args.charAt(i);
				if (ch == ' ')
					nargs.append("%20");
				else
					nargs.append(ch);
			}
			URL url = new URL(scriptname + nargs.toString());
			//in = new DataInputStream(url.openStream());
			ioerr = true;
			if (encoding != null && !ioerr) {
				try {
					in =
						new BufferedReader(
							new InputStreamReader(url.openStream(), encoding)); //CHANGED , encoding)); By Asad
					System.out.println(
						"Success connecting to ["
							+ scriptname
							+ args
							+ "] with encoding "
							+ encoding);
				} catch (Exception e) {
					System.out.println(
						"Encoding [" + encoding + "] not supported!");
					e.printStackTrace();
					ioerr = true;
				}
			}
			if (ioerr || encoding == null) {
				try {
					in =
						new BufferedReader(
							new InputStreamReader(url.openStream(), encoding));// changed, "8859_1")); By asad
					System.out.println(
						"Success connecting to [" + scriptname + args + "]");
				} catch (Exception e) {
					System.out.println(
						"Encoding [" + encoding + "] not supported!");
					e.printStackTrace();
				}
			}
		} catch (Exception e) {
			System.out.println(
				"Exception connecting to ["
					+ scriptname
					+ args
					+ "]"
					+ ": "
					+ e);
		}
	}
	/** Disconnects from the server (closes the connection) */
	public void close() throws IOException {
		in.close();
	}
	/** Returns the HighScore denoted by the string s
	 * @see visl.rasmus.hitserver.server.HitServerInterface */
	public Vector getHighScore(String s) {
		Vector v = new Vector();
		try {
			connect(
				"?language="
					+ language
					+ "&game="
					+ game
					+ "&command=GETHIGHSCORE&arg="
					+ s);
			String line = readEnc();
			while (!line.equals(EOT)) {
				if (line.startsWith("HighScoreItem"))
					v.addElement(new HighScoreItem(line));
				line = readEnc();
			}
			close();
		} catch (Exception e) {
			System.out.println("Exception while retreiving highscore: " + e);
		}
		return v;
	}
	/** Returns the overall position on the highscore given the score and
	 * hitrate (on the T100Y highscore) */
	public int getPosition(int score, float hitrate) {
		int r = -1;
		try {
			connect(
				"?language="
					+ language
					+ "&game="
					+ game
					+ "&command=GETHIGHSCOREPOS&hashcode="
					+ ((int) (score * 100 + hitrate)));
			r = Integer.parseInt(readEnc());
			close();
		} catch (Exception e) {
			System.out.println(
				"Exception while retrieving highscore position:" + e);
		}
		return r;
	}
	/** Returns a vector with num random sentences that all contains at least
	 * one word with the form <CODE>form</CODE> and all has at least 
	 * <CODE>min</CODE> words and less than <CODE>max</CODE> words. */
	public Vector getRandomSentencesWithForm(
		String form,
		int min,
		int max,
		int num) {
		Vector res = new Vector();
		int ix = 0;
		try {
			form = oldString(form);
			connect(
				"?language="
					+ language
					+ "&command=RANDOMSENTENCESWITHFORM&forms="
					+ form
					+ "&min="
					+ min
					+ "&max="
					+ max
					+ "&num="
					+ num);
			String line = readEnc();
			while (!line.startsWith(EOT)) {
				//System.out.println("Debug{"+line+"}");
				if (line.startsWith("Sentence[")) {
					Sentence s = new Sentence(line);
					/*
					for (int i = 0; i < s.size(); i++) {
					Word w = s.getWord(i);
					w.text = newString(w.text);
					w.form = newString(w.form);
					}
					*/
					res.addElement(new Sentence(line));
				}
				line = readEnc();
			}
			close();
		} catch (Exception e) {
			System.out.println("Exception while retreiving sentences:" + e);
			System.out.println("res = " + res + ", |res| = " + res.size());
			res = null;
		}
		return res;
	}
	/** Returns the <CODE>num</CODE> sentence in the current language 
	 * @see visl.rasmus.hitserver.data.Sentence */
	public Sentence getSentence(int num) {
		Sentence res = null;
		try {
			connect("?language=" + language + "&command=GETSENTENCE&id=" + num);
			String line = readEnc();
			res = new Sentence(line);
			close();
		} catch (Exception e) {
			System.out.println("Exception while retreiving sentence:" + e);
		}
		return res;
	}
	/** Returns a random sentence from the current language
	 * @see visl.rasmus.hitserver.data.Sentence */
	public Sentence getRandomSentence() {
		Sentence res = null;
		try {
			connect("?language=" + language + "&command=RANDOMSENTENCE");
			String line = readEnc();
			res = new Sentence(line);
			close();
		} catch (Exception e) {
			System.out.println("Exception while retreiving sentence:" + e);
		}
		return res;
	}
	/** Returns a random sentence with min min words and max max words 
	 * @see visl.rasmus.hitserver.data.Sentence */
	public Sentence getRandomSentence(int min, int max) {
		Sentence res = null;
		try {
			connect(
				"?language="
					+ language
					+ "&command=RANDOMSENTENCE&min="
					+ min
					+ "&max="
					+ max);
			String line = readEnc();
			res = new Sentence(line);
			close();
		} catch (Exception e) {
			System.out.println("Exception while retreiving sentence:" + e);
		}
		return res;
	}
	/** Returns a vector containing <CODE>num</CODE> random trees that satisfy
	 * the criteria that there is atleast <CODE>min</CODE> and no more 
	 * than <CODE>max</CODE> words in the tree (if less than <CODE>num</CODE>
	 * trees satisfies the criteria, the vector is truncated)
	 * @see visl.rasmus.hitserver.data.TreeNode */
	public Vector getRandomTrees(int min, int max, int num) {
		Vector res = new Vector();
		try {
			TreeNode nn = new TreeNode();
			connect(
				"?command=RANDOMTREES&language="
					+ language
					+ "&min="
					+ min
					+ "&max="
					+ max
					+ "&num="
					+ num);
			//out.println(GET_TREES+language+";"+game+";"+min+";"+max+";"+num);
			String line = readEnc();
			while (!line.equals(EOT)) {
				//System.out.println("Got: " + line);
				if (line.startsWith("Tree[")) {
					//System.out.println(line);
					TreeNode n = TreeNode.buildTree(line);
					res.addElement(n);
				}
				line = readEnc();
			}
			close();
		} catch (Exception x) {
			System.out.println("Exception while retreiving tree: " + x);
		}
		return res;
	}
	/** Adds to the highscore of the current language in the current game
	 * with the name <CODE>name</CODE> and the score <CODE>score</CODE> */
	public void putHighScore(
		String name,
		String country,
		int score,
		float hitrate) {
		try {
			name = name.trim();
			country = country.trim();
			name = name.replace(' ', '+');
			country = country.replace(' ', '+');
			connect(
				"?language="
					+ language
					+ "&game="
					+ game
					+ "&command=PUTHIGHSCORE&name="
					+ name
					+ "&country="
					+ country
					+ "&score="
					+ score
					+ "&hitrate="
					+ hitrate);
			close();
		} catch (Exception e) {
			System.out.println(
				"Exception while trying to add to HighScore:" + e);
		}
	}
	/** Returns a vector containing strings that denotes all the wordclasses
	 * in the current language. 
	 * @param wc can be "all", "minor", "major", "nv". */
	public Vector getWordClasses(String wc) {
		Vector res = new Vector();
		try {
			connect(
				"?command=GETWORDCLASSES&language=" + language + "&wc=" + wc);
			//out.println(GET_WCS+language+";"+wc);
			
			
			// ******************************************
			// Not reading encoding here, reading line of Word Clases here
			// the name of method is ambigious
			
			String s = readEnc();
			
			// s = :n:v:adj:adv:art:pron:prp:conj:num:infm:intj:
			close();
			int ix = s.indexOf(':') + 1;
			int jx = s.indexOf(':', ix);
			while (ix < s.length() - 1) {
				String tmp = s.substring(ix, jx);
				System.out.println(tmp);
				if (tmp.length() > 0) {
					res.addElement(tmp);
					ix = jx + 1;
					jx = s.indexOf(':', ix);
				}
			}
		} catch (Exception e) {
			System.out.println(
				"HitClient:getWordClasses(" + language + ":" + e + ")");
		}
		return res;
	}
	/** Returns a vector containing the functions
	 * @param desc determines whether the format should be "short" or "long" */
	public Vector getFunctions(String desc) {
		Vector res = new Vector();
		try {
			connect(
				"?command=GETFUNCTIONS&language=" + language + "&desc=" + desc);
			//out.println(GET_FUNCS+language+";"+desc);
			String s = readEnc();
			close();
			int ix = s.indexOf('|') + 1;
			int jx = s.indexOf('|', ix);
			while (ix < s.length() - 1) {
				res.addElement(s.substring(ix, jx));
				ix = jx + 1;
				jx = s.indexOf('|', ix);
			}
		} catch (Exception e) {
			System.out.println(
				"HitClient:getFunctions(" + language + ":" + e + ")");
		}
		return res;
	}
	/** Returns all the languages that are supported by the database. */
	public String[] getLanguages() {
		String res[] = null;
		Vector v = new Vector();
		try {
			connect("?command=GETLANGUAGES");
			String s = readEnc();
			while (!s.equals(EOT)) {
				v.addElement(s);
				s = readEnc();
			}
			close();
			res = new String[v.size()];
			for (int i = 0; i < res.length; i++)
				res[i] = (String) v.elementAt(i);
		} catch (Exception x) {
		}
		return res;
	}
	/** Returns all the languages that are supported by the current game. */
	public String[][] getLanguages4Game() {

		Vector v = sendQuery("?command=GETLANGUAGES4GAME&game=" + game);
		String res[][] = null;
		if (v.size() > 0) {
			res = new String[v.size()][2];
			for (int i = 0; i < res.length; i++) {
				String s = (String) v.elementAt(i);
				res[i][0] = s.substring(0, s.indexOf(" & "));
				res[i][1] = s.substring(s.indexOf(" & ") + 3);
			}
		}
		return res;
	}
	/** Returns all the possible games */
	public String[] getGames() {
		Vector v = sendQuery("?command=GETGAMES&language=" + language);

		Vector v2 = new Vector();
		for (int i = 0; i < v.size(); i++) {
			String val = (String) v.elementAt(i);
			String s = "";
			int ix = val.indexOf(":") + 1;
			int jx = val.indexOf(":", ix);
			while (jx != -1) {
				s = val.substring(ix, jx);
				if (!v2.contains(s))
					v2.addElement(s);
				ix = jx + 1;
				jx = val.indexOf(":", ix);
			}
		}
		String res[] = new String[v2.size()];
		for (int i = 0; i < v2.size(); i++)
			res[i] = (String) v2.elementAt(i);
		return res;
	}
	/** Returns the encoding of the current language. 
	 * NB: If none is present in the database null is returned */
	public String getEncoding() {
		
		/*
		 CHANEGD BY ASAD
		 Stop reading Encoding from DB, forced to be UTF 
		 */
		/*try {
			connect("?command=GETENCODING&language=" + language);
			
			String line = readEnc();
			//System.out.println("line = " + line);
			if (!line.equals(EOT))
				encoding = line;
			else
				encoding = null;
		} catch (Exception x) {
			System.out.println("Exception while retreiving encoding : " + x);
			x.printStackTrace();
			encoding = null;
		}*/
		return encoding;
	}
	/** Sends a query-command to the HitServer. Returns a vector
	 * containing all the lines that are returned. */
	protected Vector sendQuery(String command) {
		Vector res = new Vector();
		try {
			connect(command);
			String line = readEnc();
			while (!line.equals(EOT)) {
				res.addElement(line);
				line = readEnc();
			}
			close();
		} catch (Exception x) {
			System.out.println(
				"Exception while trying to execute command ["
					+ command
					+ "]: "
					+ x);
			x.printStackTrace();
		}
		return res;
	}
	// ***********************************************************************
	// * Field Methods
	// ***********************************************************************
	/** Changes the language used */
	public void setLanguage(String lang) {
		language = lang;
		encoding =  "UTF-8"; // CHANGED getEncoding(); by Asad
		System.out.println(
			"Set language to " + lang + ", new encoding is " + encoding);
	}
    /** 
     * @return the language used. 
     */
    public String getLanguage(){
	return language;
    }
	/** Changes the game parameter */
	public void setGame(String gam) {
		game = gam;
	}
	public void setPattern(String pat) {
		pattern = pat;
	}
	
	
	public String newString(String str) {
		if (encoding == null)
			return str;
		String s = str;
		try {
			//  CHANGED s = new String(str.getBytes("8859_1"), encoding); by Asad
			s = new String(str.getBytes("UTF-8"), encoding);
		} catch (Exception e) {
			//System.out.println("8859_1 => " + encoding + " is not supported!");
			System.out.println("UTF_8 => " + encoding + " Problem in Conversion!");
			s = str;
			e.printStackTrace();
		}
		return s;
	}
	public String oldString(String str) {
			if (encoding == null)
			return str;
		String s = str;
		try {
			// CNAHGED s = new String(str.getBytes(encoding), "8859_1"); By Asad
			s = new String(str.getBytes(encoding), "UTF-8");
		} catch (Exception e) {
			System.out.println(encoding + " => UTF-8 problem in conversion!");
			s = str;
			e.printStackTrace();
		}
		return s;
	}
}
