package visl.rasmus.hitserver.data;

import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

public class HighScore {
    public static final long 
	DAY_MILLIS = 1000*60*60*24,
	WEEK_MILLIS = DAY_MILLIS*7,
	MONTH_MILLIS = DAY_MILLIS*31,
	YEAR_MILLIS = DAY_MILLIS*365;
    public static final int MAX_PLAYERS = 440;
    protected Vector items;
    public HighScore() {
	items = new Vector();
    }
    public int add(HighScoreItem hsi) {
	int r = getPosition(hsi.hashCode());
	if (r != -1) {
	    items.insertElementAt(hsi, r);
	}
	return r;
    }
    public int getPosition(int hashcode) {
	int ix = 0;
	int jx = 0, tx = -1;
	HighScoreItem i1;
	synchronized (items) {
	    int n = items.size();
	    if (n >= MAX_PLAYERS) cleanUp();
	    HighScoreItem i2;
	    if (n == 0) {
		ix = 0;
	    } else if (n == 1) {
		i1 = (HighScoreItem)items.elementAt(ix);
		if (hashcode <= i1.hashCode()) ix = 1;
	    } else {
		jx = n - 1;
		i1 = (HighScoreItem)items.elementAt(ix);
		i2 = (HighScoreItem)items.elementAt(jx);
		if (hashcode > i1.hashCode()) ix = 0;
		else if (hashcode < i2.hashCode()) ix = jx+1;
		else {
		    tx = ix + (jx-ix)/2;
		    while (ix != tx) {
			i1 = (HighScoreItem)items.elementAt(tx);
			if (hashcode > i1.hashCode()) jx = tx;
			else ix = tx;
			tx = ix + (jx - ix)/2;
		    }
		    i1 = (HighScoreItem)items.elementAt(ix);
		    if (hashcode <= i1.hashCode()) ix = ix + 1;
		}
	    }
	}
	return ix;
    }
    /** Clears up all the old items + truncates the highscore */
    public void cleanUp() {
	synchronized (items) {
	    int pos = 0;
	    Date d100y = new Date(System.currentTimeMillis()-YEAR_MILLIS);
	    Date d100m = new Date(System.currentTimeMillis()-MONTH_MILLIS);
	    Date d10w = new Date(System.currentTimeMillis()-WEEK_MILLIS);
	    Date d10d = new Date(System.currentTimeMillis()-DAY_MILLIS);
	    HighScoreItem hsi = null;
	    for (Enumeration en = items.elements(); en.hasMoreElements();) {
		hsi = (HighScoreItem)en.nextElement();
		if (hsi.date.before(d100y)) items.removeElement(hsi);
		else if (hsi.date.before(d100m) && pos >= 100) 
		    items.removeElement(hsi);
		else if (hsi.date.before(d10w) && pos >= 200) 
		    items.removeElement(hsi);
		else if (hsi.date.before(d10d) && pos >= 210)
		    items.removeElement(hsi);
		else pos++;
		
	    }
	    if (items.size() > MAX_PLAYERS) {
		items.setSize(MAX_PLAYERS);
		items.trimToSize();
	    }
	}
    }
    public Enumeration getAllItems() { return items.elements(); }
    public Enumeration getTop100_Year() {
	Date d = new Date(System.currentTimeMillis()-YEAR_MILLIS);
	return new AfterDate(d, 100);
    }
    public Enumeration getTop10_Year() {
	Date d = new Date(System.currentTimeMillis()-YEAR_MILLIS);
	return new AfterDate(d, 10);	
    }
    public Enumeration getTop100_Month() {
	Date d = new Date(System.currentTimeMillis()-MONTH_MILLIS);
	return new AfterDate(d, 100);	
    }
    public Enumeration getTop10_Month() {
	Date d = new Date(System.currentTimeMillis()-MONTH_MILLIS);
	return new AfterDate(d, 10);	
    }
    public Enumeration getTop10_Week() {
	Date d = new Date(System.currentTimeMillis()-WEEK_MILLIS);
	return new AfterDate(d, 10);	
    }
    public Enumeration getTop10_Day() {
	Date d = new Date(System.currentTimeMillis()-DAY_MILLIS);
	return new AfterDate(d, 10);	
    }
    public int size() { return items.size(); }
    /** Returns the next index of the highscore which references an item newer
     * than date d and after index i, if none is found returns -1 */
    protected int getNextIndex(Date d, int i) {
	int ix = i;
	int n = items.size();
	if (ix >= n) return -1;
	HighScoreItem hsi = (HighScoreItem)items.elementAt(ix);
	while (hsi.date.before(d) && ix < n-1) {
	    ix++;
	    hsi = (HighScoreItem)items.elementAt(ix);
	}
	if (ix >= n) ix = -1;
	return ix;
    }
    // ***********************************************************************
    // * Enumeration Classes
    // ***********************************************************************
    public class AfterDate implements Enumeration {
	protected Date date;
	protected int num, ix;
	public AfterDate(Date date, int num) {
	    this.date = date;
	    this.num = num;
	    this.ix = getNextIndex(date, 0);
	}
	public boolean hasMoreElements() {
	    return (num > 0 && ix != -1);
	}
	public Object nextElement() {
	    Object o = null;
	    synchronized (items) {
		o = items.elementAt(ix);
		num--;
		if (num > 0) ix = getNextIndex(date, ix + 1);
		else ix = -1;
	    }
	    return o;
	}
    }
    public boolean save(String filename) {
	boolean success = false;
	try {
	    FileOutputStream fout = new FileOutputStream(filename);
	    PrintStream out = new PrintStream(fout);
	    for (Enumeration en = items.elements(); en.hasMoreElements();) {
		HighScoreItem hsi = (HighScoreItem)en.nextElement();
		if (hsi != null) out.println(hsi);
	    }
	    fout.close();
	    success = true;
	} catch (Exception x) {
	    System.out.println("Exception while trying to save highscore to " +
			       filename + ": " + x);
	    success = false;
	}
	return success;
    }
    public boolean load(String filename) {
	items.removeAllElements();
	boolean success = false;
	try {
	    FileInputStream fin = new FileInputStream(filename);
	    DataInputStream in = new DataInputStream(fin);
	    String line = in.readLine();
	    while (line != null) {
		add(new HighScoreItem(line));
		line = in.readLine();
	    }
	    fin.close();
	    success = true;
	} catch (Exception x) {
	    System.out.println("Exception while trying to load highscore from " +
			       filename + ": " + x);
	    success = false;
	}
	return success;
    }
}
