package visl.rasmus.hitserver.data;

import java.util.Vector;

public class Sentence {
    protected Vector words;
    public Sentence() {
	words = new Vector();
    }
    /** Generates a sentence based on a string representation of the sentence
     * Syntax: Sentence[word0:form0;word1:form1...]
     */
    public Sentence(String s) {
	words = new Vector();
	try {
	    int ix = s.indexOf("[");
	    if (s.substring(0,ix).equals("Sentence")) {
		int jx = ix + 1;
		ix = getNextIndex(s, ':', jx);
		String text, form;
		int l = s.length();
		boolean eos = false;
		while (!eos) {
		    text = s.substring(jx, ix);
		    jx = ix + 1;
		    ix = getNextIndex(s, ';', jx);
		    if (ix == -1) {
			ix = s.indexOf("]", jx);
			eos = true;
		    }
		    form = s.substring(jx, ix);
		    jx = ix + 1;
		    ix = getNextIndex(s, ':', jx);
		    addWordX(text, form);
		}
	    } else throw new RuntimeException("Not the correct classname");
	} catch (Exception x) {
	    System.out.println("Sentence:Exception when resolving data["+
			       s+"]: " + x);
	    x.printStackTrace();
	}
    }
    public void addWord(String text, String form) {
	words.addElement(new Word(text, form));
    }
    public void addWordX(String text, String form) {
	words.addElement(new Word(deEscape(text), deEscape(form)));
    }
    public Word getWord(int i) { return (Word)words.elementAt(i); }
    public int size() { return words.size(); }
    public boolean containsForm(String form) {
	boolean res = false;
	for (int i = 0; i < words.size() && !res; i++) {
	    Word w = (Word)words.elementAt(i);
	    res = (form.indexOf("|"+w.form+"|") != -1);
	}
	return res;
    }
    public String toString() {
	StringBuffer buff = new StringBuffer("Sentence[");
	int l = size();
	for (int i = 0; i < l; i++) {
	    buff.append(getWord(i));
	    if (i != l-1) buff.append(";");
	}
	buff.append("]");
	return buff.toString();
    }
    public boolean equals(Object o) {
	if (!(o instanceof Sentence)) return false;
	Sentence s = (Sentence)o;
	return toString().equals(s.toString());
    }
    /** Converts the sentence into a unique string of max 255 characters
     * (for use with MySQL as primary key */
    public String uniqueString() {
	StringBuffer buff = new StringBuffer();
	buff.append(String.valueOf(size()));
	String s = toString();
	s = s.substring(s.indexOf('[')+1, s.lastIndexOf(']'));
	for (int i = 0; i < s.length(); i++) {
	    // Filter out delimiters and escape-characters
	    if (s.charAt(i) != ';' &&
		s.charAt(i) != ':' &&
		s.charAt(i) != '"' &&
		s.charAt(i) != '\'') buff.append(s.charAt(i));
	}
	// Make sure the length does not succeed 255
	//if (buff.length() > 255) System.out.println("Unique string too long: "+
	//buff);
	//buff.setLength(255);
	return buff.toString();
    }
    /** Returns the index of the next occurence of <CODE>sep</CODE> that has
     * not been escaped by the '+' escape character. */
    static public int getNextIndex(String s, char sep, int offs) {
	int r = s.indexOf(sep, offs);
	if (r == 0) return r;
	while (r > 0 && s.charAt(r-1) == '+') {
	    //r = s.indexOf(sep, r);
	    r = s.indexOf(sep, r+1);
	}
	return r;
    }
    public static String deEscape(String s) {
	StringBuffer buff = new StringBuffer();
	for (int i = 0; i < s.length(); i++) {
	    if ((s.charAt(i) == ':' ||
		 s.charAt(i) == '+')) i++;
	    buff.append(s.charAt(i));
	}
	return buff.toString();
    }
    public static String escape(String s) {
	StringBuffer buff = new StringBuffer(s);
	int cnt = 0;
	while (cnt < buff.length()) {
	    if (buff.charAt(cnt) == ':' ||
		buff.charAt(cnt) == '+')
		buff.insert(cnt++, '+');
	    cnt++;
	}
	//if (s.length() != buff.length()) System.out.println("Escaped ["+s+"]->["+buff.toString()+"]");
	return buff.toString();
    }
    public static void main(String args[]) {
	if (args.length != 1) {
	    System.out.println("Usage: visl.rasmus.hitserver.data.Sentence <sentence>");
	} else {
	    System.out.println("Trying to create sentence ["+args[0]+"]");
	    Sentence s = new Sentence(args[0]);
	    for (int i = 0; i < s.size(); i++) {
		System.out.println("no. "+ i +" Word [" + s.getWord(i).text +"]:[" + s.getWord(i).form+"]");
	    }
	}
    }
}






