package visl.rasmus.hitserver.gui;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Event;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import visl.rasmus.gui.Graph;
import visl.rasmus.hitserver.client.HitXtraClient;
import visl.rasmus.hitserver.server.HitServerInterface;

public class DebugDatabases extends Frame {
    protected Frame parent;
    protected TextArea display;
    protected java.awt.List languages, games;
    protected CheckboxGroup cbg_db;
    protected HitXtraClient hc;
    protected Graph graph;
    public DebugDatabases(Frame parent, HitXtraClient hc) {
	super("Debug the Databases");
	this.hc = hc;
	this.parent = parent;
	display = new TextArea();
	add("Center", display);
	graph = null;
	Panel p = new Panel();
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 1;
	c.weighty = 1;
	p.setLayout(gbl);
	Label l = null;
	Checkbox cb = null;

	c.gridx = 0;
	c.gridy = 0;
	c.gridheight = 2;
	l = new Label("Language:");
	gbl.setConstraints(l, c);
	p.add(l);
	String sa[] = hc.getLanguages();
	languages = new java.awt.List(2,false);
	for (int i = 0; i < sa.length; i++)
	    languages.addItem(sa[i]);
	c.gridx = 1;
	gbl.setConstraints(languages, c);
	p.add(languages);
	
	c.gridx = 0;
	c.gridy = 2;
	l = new Label("Games:");
	gbl.setConstraints(l, c);
	p.add(l);
	sa = hc.getGames();
	games = new java.awt.List(2,false);
	for (int i = 0; i < sa.length; i++)
	    games.addItem(sa[i]);	
	c.gridx = 1;
	gbl.setConstraints(games, c);
	p.add(games);

	c.gridheight = 1;
	c.gridx = 2;
	c.gridy = 0;
	cbg_db = new CheckboxGroup();
	cb = new Checkbox("SentenceDB", cbg_db, true);
	gbl.setConstraints(cb, c);
	p.add(cb);
	cb = new Checkbox("HighScoreDB", cbg_db, false);
	c.gridy = 1;
	gbl.setConstraints(cb, c);
	p.add(cb);
	cb = new Checkbox("SettingsDB", cbg_db, false);
	c.gridy = 2;
	gbl.setConstraints(cb, c);
	p.add(cb);
	cb = new Checkbox("TreeDB", cbg_db, false);
	c.gridy = 3;
	gbl.setConstraints(cb, c);
	p.add(cb);
	add("North", p);
	p = new Panel();
	gbl = new GridBagLayout();
	c = new GridBagConstraints();
	p.setLayout(gbl);
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 1;
	c.weighty = 1;
	Button b = null;

	c.gridwidth = 1;
	l = new Label("Graphs:");
	gbl.setConstraints(l, c);
	p.add(l);
	b = new Button("List all forms");
	gbl.setConstraints(b, c);
	p.add(b);
	b = new Button("Show frequency");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(b, c);
	p.add(b);

	c.gridwidth = 1;
	l = new Label("Commands:");
	gbl.setConstraints(l, c);
	p.add(l);
	c.gridwidth = GridBagConstraints.REMAINDER;
	b = new Button("Quit");
	gbl.setConstraints(b, c);
	p.add(b);

	add("South", p);
	pack();
	show();
	parent.hide();
    }
    public boolean handleEvent(Event e) {
	if (e.id == Event.WINDOW_DESTROY) {
	    hide();
	    dispose();
	    parent.show();
	    parent.toFront();
	    return true;
	}
	if (e.id == Event.ACTION_EVENT && e.target instanceof Button) {
	    Button b = (Button)e.target;
	    // Remove graph before anything else.
	    if (graph != null) {
		remove(graph);
		graph = null;
		add("Center", display);
		validate();
	    }
	    if (b.getLabel().equals("Quit")) {
		postEvent(new Event(this, Event.WINDOW_DESTROY, null));
		return true;
	    } else if (b.getLabel().equals("List all forms")) {
		doListForms();
		return true;
	    } else if (b.getLabel().equals("Show frequency")) {
		doShowLengthFrequency();
		return true;
	    }
	}
	return false;
    }
    protected void doListForms() {
	String lang = languages.getSelectedItem();
	display.setText("");
	if (lang == null) {
	    display.append("You have to choose a language!\n");
	    return;
	}
	Checkbox cb = cbg_db.getCurrent();
	int db = (cb.getLabel().equals("SentenceDB") ? 0 :
		  (cb.getLabel().equals("HighScoreDB") ? 1 :
		   (cb.getLabel().equals("SettingsDB") ? 2 : 3)));
	Hashtable hash = new Hashtable();
	int x[] = null;
	int y[] = null;
	String legend[] = null;
	String title = null;
	switch (db) {
	case 0:
	    Enumeration en = hc.sendQuery(HitServerInterface.SQL_COMMAND+"0:q:SELECT forms "+
					  "FROM " + lang).elements();
	    while (en.hasMoreElements()) {
		Vector v2 = string2Vector((String)en.nextElement(), ":");
		for (int i = 0; i < v2.size(); i++)
		    if (!hash.containsKey(v2.elementAt(i)))
			hash.put(v2.elementAt(i), new Integer(1));
		    else {
			hash.put(v2.elementAt(i), new Integer(((Integer)hash.get(v2.elementAt(i))).intValue()+1));
		    }
	    }
	    int cnt = 0;
	    legend = new String[hash.size()];
	    x = new int[hash.size()];
	    y = new int[hash.size()];
	    title = "Number of occurrences of the various forms.";
	    /*
	    String keys[] = new String[hash.size()];
	    int indicies[] = new int[hash.size()];
	    Integer vals[] = new Integer[hash.size()];
	    for (int i = 0; i < indicies.length; i++) indicies[i] = i;
	    */
	    Vector vec = new Vector();
	    for (en = hash.keys(); en.hasMoreElements(); ) {
		String s = (String)en.nextElement();
		vec.addElement(new FreqElement((Integer)hash.get(s), s));
	    }
	    //QSortAlgorithm.sort(vals, indicies);
	    Collections.sort(vec);
	    String wc_all = ":";
	    int n = vec.size()-1;
	    for (int i = 0; i <= n; i++) {
		FreqElement fe = (FreqElement)vec.elementAt(n-i);
		x[i] = i;
		y[i] = fe.val.intValue();
		legend[i] = fe.key;
		wc_all = ":" + fe.key + wc_all;
	    }
	    System.out.println("wc_all = " + wc_all);
	    break;
	case 1:
	    display.append("This function has not yet been implemented for\n"+
			   "the HighScoreDB!");
	    break;
	case 2:
	    display.append("This function has not yet been implemented for\n"+
			   "the SettingsDB!");
	    break;
	case 3:
	    display.append("This function has not yet been implemented for\n"+
			   "the TreeDB!");
	    break;
	}
	/*display.append("Forms from the " + cbg_db.getCurrent().getLabel() + 
		       ":\n");
	for (int i = 0; i < v.size(); i++) {
	    display.append(v.elementAt(i)+"\n");
	}
	*/
	if (x != null) {
	    if (display.getParent() != null) {
		remove(display);
	    }
	    if (graph == null) {
		graph = new Graph(title, x, y, legend);
		add("Center", graph);
		validate();
	    } else {
		graph.setTitle(title);
		graph.setXY(x, y);
	    }
	    if (graph != null) graph.repaint();
	}
    }
    /** Converts the string s containing a sep separated list of items into
     * a vector */
    protected Vector string2Vector(String s, String sep) {
	Vector v = new Vector();
	int ix, jx;
	ix = s.indexOf(sep);
	if (ix != 0) {
	    jx = ix;
	    ix = 0;
	} else jx = s.indexOf(sep, ix);
	while (jx != -1) {
	    v.addElement(s.substring(ix, jx));
	    ix = jx + sep.length();
	    jx = s.indexOf(sep, ix);
	}
	if (ix < s.length()-1) v.addElement(s.substring(ix));
	return v;
    }
    protected void doShowLengthFrequency() {
	String lang = languages.getSelectedItem();
	display.setText("");
	if (lang == null) {
	    display.append("You have to choose a language!\n");
	    return;
	}
	Checkbox cb = cbg_db.getCurrent();
	int db = (cb.getLabel().equals("SentenceDB") ? 0 :
		  (cb.getLabel().equals("HighScoreDB") ? 1 :
		   (cb.getLabel().equals("SettingsDB") ? 2 : 3)));
	int x[] = null;
	int y[] = null;
	String title = "Title";
	switch (db) {
	case 0:
	    title = "Frequency vs. Length of sentences";
	    Enumeration en = hc.sendQuery(HitServerInterface.SQL_COMMAND+"0:q:SELECT length, "+
					  "COUNT(length) FROM " + lang +
					  " GROUP BY length").elements();
	    Vector vx = new Vector();
	    Vector vy = new Vector();
	    while (en.hasMoreElements()) {
		try {
		    String s = (String)en.nextElement();
		    vx.addElement(new Integer(s.substring(0, s.indexOf(" & "))));
		    vy.addElement(new Integer(s.substring(s.indexOf(" & ")+3)));
		} catch (NumberFormatException _) {
		    _.printStackTrace();
		}
	    }
	    x = new int[vx.size()];
	    y = new int[x.length];
	    for (int i = 0; i < x.length; i++) {
		x[i] = ((Integer)vx.elementAt(i)).intValue();
		y[i] = ((Integer)vy.elementAt(i)).intValue();
	    }
	    break;
	case 1:
	    return;
	case 2:
	    break;
	case 3:
	    break;
	}
	if (x != null) {
	    if (display.getParent() != null) {
		remove(display);
	    }
	    if (graph == null) {
		graph = new Graph(title, x, y);
		add("Center", graph);
		validate();
	    } else {
		graph.setTitle(title);
		graph.setXY(x, y);
	    }
	    if (graph != null) graph.repaint();
	}
    }
}

