package visl.rasmus.hitserver.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Component;
import java.awt.Event;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.util.Enumeration;
import java.util.Vector;

import visl.rasmus.gui.InputDialog;
import visl.rasmus.gui.MessageDialog;
import visl.rasmus.gui.ScrollablePanel;
import visl.rasmus.gui.Separator;
import visl.rasmus.gui.YesNoDialog;
import visl.rasmus.hitserver.client.HitXtraClient;
import visl.rasmus.hitserver.server.HitServerInterface;

public class ManageLanguages extends Frame {
    protected HitXtraClient hc;
    protected Frame parent;
    protected Panel root, current;
    protected Checkbox wcs[], gms[], fcs[];
    protected TextField name, lang_path, f_short[], f_long[], encoding;
    protected String language;
    protected boolean d_sdb, d_enc, d_gms, d_wcs, d_fcs, d_lng; // Dirty flags
    protected boolean d_nam;
    protected Button b_update;
    protected Vector languages;

    public ManageLanguages(int port, Frame parent) {
	hc = new HitXtraClient("localhost", port, null, null);
	this.parent = parent;

	gms = null;
	wcs = null;
	fcs = null;
	name = null;
	lang_path = null;
	encoding = null;
	f_short = null;
	f_long = null;

	root = new Panel();
	root.setLayout(new BorderLayout());
	Label l = null;
	Panel p = null;
	Button b = null;

	languages = getLanguages();
	d_lng = false;
	language = (String)languages.firstElement();
	current = new Panel();
	current.resize(100,100);
	layoutLanguages(languages, true);

	//ScrollablePanel sp = new ScrollablePanel(root);
	add("Center", root);

	p = new Panel();
	b = new Button("Quit");
	p.add(b);
	b_update = new Button("Update Database");
	p.add(b_update);
	b = new Button("Refresh from Database");
	p.add(b);
	b = new Button("Add Language");
	p.add(b);
	b = new Button("Remove Language");
	p.add(b);
	add("South", p);
    }
    public boolean handleEvent(Event e) {
	if (e.id == Event.WINDOW_DESTROY) {
	    this.dispose();
	    parent.show();
	    return true;
	}
	// NOTE: A Tiny little bit of a HACK!!
	if (e.target instanceof Button && 
	    e.id == Event.ACTION_EVENT &&
	    ((Button)e.target).getParent().getParent() == root) {
	    Button b = (Button)e.target;
	    if (changeLanguage()) {
		language = b.getLabel();
		if (d_lng) languages = getLanguages();
		d_lng = false;
		layoutLanguages(languages, true);
	    }
	    return true;
	} else if (e.target instanceof Button &&
		   e.id == Event.ACTION_EVENT) {
	    Button b = (Button)e.target;
	    if (b.getLabel().equals("Quit")) {
		postEvent(new Event(this, Event.WINDOW_DESTROY, null));
		return true;
	    } else if (b.getLabel().equals("Update Database")) updateLanguage();
	    else if (b.getLabel().equals("Refresh from Database")) refresh();
	    else if (b.getLabel().equals("Add Language")) addLanguage();
	    else if (b.getLabel().equals("Remove Language")) removeLanguage();
	    else if (b.getLabel().equals("Add Function")) addFunction();
	    else if (b.getLabel().equals("Remove Selected Functions")) 
		removeFunctions();
	    else if (b.getLabel().equals("Reload Sentences"))
		hc.sendCommand(HitServerInterface.RELOAD_SENTENCES+language);
	    else if (b.getLabel().equals("Reload Trees"))
		hc.sendCommand(HitServerInterface.RELOAD_TREES+language);
	    else if (b.getLabel().equals("Reload All Sentences"))
		hc.sendCommand(HitServerInterface.RELOAD_SENTENCES+"ALL");
	    else if (b.getLabel().equals("Reload All Trees"))
		hc.sendCommand(HitServerInterface.RELOAD_TREES+"ALL");
	    else if (b.getLabel().equals("Add Game"))
		doAddGame();
	    else if (b.getLabel().equals("Remove Game(s)")) 
		doRemoveGames();
	    else if (b.getLabel().equals("Select All"))
		doSelectAllGames();
	    else if (b.getLabel().equals("Deselect All"))
		doDeselectAllGames();
	    else return false;
	    return true;
	} else if (e.target instanceof TextField &&
		   e.id == Event.KEY_PRESS) {
	    // Update the dirty-flags etc.
	    TextField tf = (TextField)e.target;
	    if (tf == lang_path) d_sdb = true;
	    else if (tf == encoding) d_enc = true;
	    else if (tf == name) d_nam = true;
	    else if (contains(f_short, tf) || contains(f_long, tf)) {
		d_fcs = true;
		Component cs[] = current.getComponents();
		int ix = getIndex(cs, tf);
		if (!(cs[ix-1] instanceof Checkbox)) ix--;
		if (cs[ix - 1] instanceof Checkbox) {
		    ((Checkbox)cs[ix-1]).setState(true);
		} else {
		    System.out.println("Some error: ix = " + ix);
		}
	    }
	    validate();
	} else if (e.target instanceof Checkbox &&
		   e.id == Event.ACTION_EVENT) {
	    Checkbox cb = (Checkbox)e.target;
	    if (contains(wcs, cb)) {
		d_wcs = true;
	    } else if (contains(gms, cb)) d_gms = true;
	    validate();
	}
	return false;
    }
    protected void layoutLanguages(Vector v, boolean update) {
	root.removeAll();
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	Panel p = new Panel();
	p.setLayout(gbl);
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 1;
	c.weighty = 1;
	Button b = null;
	for (Enumeration en = v.elements(); en.hasMoreElements(); ) {
	    String s = (String)en.nextElement();
	    b = new Button(s);
	    if (s.equals(language)) b.disable();
	    if (!en.hasMoreElements()) c.gridwidth = GridBagConstraints.REMAINDER;
	    gbl.setConstraints(b, c);
	    p.add(b);
	}
	root.add("North", p);
	ScrollablePanel sp = new ScrollablePanel(current);
	root.add("Center", sp);
	if (update) updateLanguageVars();
	layoutLanguage();
	root.validate();
    }
    protected void initLanguageVars() {
	name = new TextField("LanguageName");
	lang_path = new TextField("[corpus-file-path ...]");
	hc.setLanguage(language);
	encoding = new TextField(hc.getEncoding());

	Vector vs[] = getWordClasses();
	wcs = new Checkbox[vs[0].size()];
	for (int i = 0; i < wcs.length; i++) {
	    String s = (String)vs[0].elementAt(i);
	    wcs[i] = new Checkbox(s, null, true);
	}

	vs = getGames();
	gms = new Checkbox[vs[0].size()];
	for (int i = 0; i < gms.length; i++) {
	    String s = (String)vs[0].elementAt(i);
	    gms[i] = new Checkbox(s, null, true);
	}

	vs = getFunctions("en");
	f_short = new TextField[vs[0].size()];
	f_long = new TextField[vs[1].size()];
	fcs = new Checkbox[f_short.length];

	for (int i = 0; i < fcs.length; i++) {
	    fcs[i] = new Checkbox(null, null, false);
	    f_short[i] = new TextField((String)vs[0].elementAt(i));
	    f_long[i] = new TextField((String)vs[1].elementAt(i));
	}

	// Clear the dirty-flags
	d_nam = true;
	d_sdb = true;
	d_enc = true;
	d_gms = true;
	d_fcs = true;
	d_wcs = true;
    }
    protected void updateLanguageVars() {
	name = new TextField(getLangName());
	lang_path = new TextField(getLangPaths());
	hc.setLanguage(language);
	encoding = new TextField(hc.getEncoding());

	Vector vs[] = getWordClasses();
	wcs = new Checkbox[vs[0].size()];
	for (int i = 0; i < wcs.length; i++) {
	    String s = (String)vs[0].elementAt(i);
	    wcs[i] = new Checkbox(s, null, vs[1].contains(s));
	}

	vs = getGames();
	gms = new Checkbox[vs[0].size()];
	for (int i = 0; i < gms.length; i++) {
	    String s = (String)vs[0].elementAt(i);
	    gms[i] = new Checkbox(s, null, vs[1].contains(s));
	}

	vs = getFunctions(language);
	f_short = new TextField[vs[0].size()];
	f_long = new TextField[vs[1].size()];
	fcs = new Checkbox[f_short.length];

	for (int i = 0; i < fcs.length; i++) {
	    fcs[i] = new Checkbox(null, null, false);
	    f_short[i] = new TextField((String)vs[0].elementAt(i));
	    f_long[i] = new TextField((String)vs[1].elementAt(i));
	}

	// Clear the dirty-flags
	d_nam = false;
	d_sdb = false;
	d_enc = false;
	d_gms = false;
	d_fcs = false;
	d_wcs = false;
    }
    /** Lays out the current language based on the instance variables language, fields, gms, wcs,
     * f_short, f_long and fcs */
    protected void layoutLanguage() {
	current.removeAll();
	//if (language == null) {
	//    return;
	//}
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	current.setLayout(gbl);
	c.weightx = 1;
	c.weighty = 1;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTHWEST;
	
	Label l = null;
	Panel p = null;
	Separator sep = null;
	Button b = null;

	l = new Label("Language:");
	c.gridwidth = 1;
	gbl.setConstraints(l, c);
	current.add(l);
	l = new Label(language);
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(l, c);
	current.add(l);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(name, c);
	current.add(name);

	sep = new Separator();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(sep, c);
	current.add(sep);

	l = new Label("corpus files:");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(l, c);
	current.add(l);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(lang_path, c);
	current.add(lang_path);
	l = new Label("encoding:");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(l,c);
	current.add(l);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(encoding, c);
	current.add(encoding);

	p = new Panel();
	b = new Button("Reload Trees");
	p.add(b);
	b = new Button("Reload Sentences");
	p.add(b);
	b = new Button("Reload All Sentences");
	p.add(b);
	b = new Button("Reload All Trees");
	p.add(b);
	gbl.setConstraints(p, c);
	current.add(p);

	sep = new Separator();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(sep, c);
	current.add(sep);

	l = new Label("WordClasses:");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(l, c);
	current.add(l);
	p = new Panel();
	p.setLayout(new GridLayout(wcs.length/8+1, 8));
	for (int i = 0; i < wcs.length; i++) {
	    p.add(wcs[i]);
	}
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(p, c);
	current.add(p);

	sep = new Separator();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(sep, c);
	current.add(sep);

	l = new Label("Games:");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(l, c);
	current.add(l);

	p = new Panel();
	p.setLayout(new GridLayout(gms.length/8+1, 8));
	for (int i = 0; i < gms.length; i++) {
	    p.add(gms[i]);
	}
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(p, c);
	current.add(p);
	p = new Panel();
	p.add(new Button("Add Game"));
	p.add(new Button("Remove Game(s)"));
	p.add(new Button("Select All"));
	p.add(new Button("Deselect All"));
	gbl.setConstraints(p, c);
	current.add(p);

	sep = new Separator();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(sep, c);
	current.add(sep);

	l = new Label("Functions");
	Font fnt = l.getFont();
	if (fnt != null)
	    l.setFont(new Font(fnt.getName(), Font.BOLD, fnt.getSize()));
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(l, c);

	current.add(l);
	p = new Panel();
	b = new Button("Add Function");
	p.add(b);
	b = new Button("Remove Selected Functions");
	p.add(b);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(p, c);
	current.add(p);

	l = new Label("Select");
	c.gridwidth = 1;
	gbl.setConstraints(l, c);
	current.add(l);
	l = new Label("Short");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(l,c);
	current.add(l);
	l = new Label("Long");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(l,c);
	current.add(l);
	
	for (int i = 0; i < fcs.length; i++) {
	    c.gridwidth = 1;
	    gbl.setConstraints(fcs[i], c);
	    current.add(fcs[i]);

	    c.gridwidth = GridBagConstraints.RELATIVE;
	    gbl.setConstraints(f_short[i], c);
	    current.add(f_short[i]);
	    
	    c.gridwidth = GridBagConstraints.REMAINDER;
	    gbl.setConstraints(f_long[i], c);
	    current.add(f_long[i]);
	}
	
	p = new Panel();
	b = new Button("Add Function");
	p.add(b);
	b = new Button("Remove Selected Functions");
	p.add(b);
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(p, c);
	current.add(p);

	current.invalidate();
	//root.validate();
	validate();
    }
    protected Vector[] getFunctions(String lang) {
	Enumeration en = hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:q:SELECT func_short, func_long FROM settings WHERE language='" + lang + "'").elements();
	Vector v1 = new Vector();
	Vector v2 = new Vector();
	while (en.hasMoreElements()) {
	    String s = (String)en.nextElement();
	    String s_sh = s.substring(0, s.indexOf(" & "));
	    String s_lg = s.substring(s.indexOf(" & ")+3);
	    int ix = s_sh.indexOf("|") + 1;
	    int jx = s_sh.indexOf("|", ix);
	    while (jx != -1) {
		v1.addElement(s_sh.substring(ix, jx));
		ix = jx + 1;
		jx = s_sh.indexOf("|", ix);
	    }
	    ix = s_lg.indexOf("|") + 1;
	    jx = s_lg.indexOf("|", ix);
	    while (jx != -1) {
		v2.addElement(s_lg.substring(ix, jx));
		ix = jx + 1;
		jx = s_lg.indexOf("|", ix);
	    }
	}
	Vector res[] = new Vector[2];
	res[0] = v1;
	res[1] = v2;
	return res;
    }
    /** Returns two identical vectors, containing the wordclasses from the
     * current language. NOTE: Used by arabic and russian. */
    protected Vector[] getWordClassesSpecial() {
	Enumeration en = 
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:q:SELECT wc_all FROM "+
			 "settings WHERE language='"+language+"'").elements();
	Vector v1 = new Vector();
	
	while (en.hasMoreElements()) {
	    String s = (String)en.nextElement();
	    int ix = s.indexOf(":") + 1;
	    int jx = s.indexOf(":", ix);
	    while (jx != -1) {
		v1.addElement(s.substring(ix, jx));
		ix = jx + 1;
		jx = s.indexOf(":", ix);
	    }
	}
	Vector res[] = new Vector[2];
	res[0] = v1;
	res[1] = v1;
	return res;
    }
    /** Returns two vectors, one containing all the possible wordclasses, and
     * one containing the wordclasses in the current language (in that order)*/
    protected Vector[] getWordClasses() {
	if (":ar:ru:".indexOf(":"+language+":") != -1) 
	    return getWordClassesSpecial();
	Enumeration en = 
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:q:SELECT language, wc_all FROM "+
			 "settings").elements();
	Vector v1 = new Vector();
	Vector v2 = new Vector();
	
	while (en.hasMoreElements()) {
	    String s = (String)en.nextElement();
	    String lng = s.substring(0, s.indexOf(" & "));
	    String val = s.substring(s.indexOf(" & ")+3);
	    int ix = val.indexOf(":") + 1;
	    int jx = val.indexOf(":", ix);
	    while (jx != -1) {
		s = val.substring(ix, jx);
		if (lng.equals(language) && !v2.contains(s)) v2.addElement(s);
		if (!v1.contains(s)) v1.addElement(s);
		ix = jx + 1;
		jx = val.indexOf(":", ix);
	    }
	}
	Vector res[] = new Vector[2];
	res[0] = v1;
	res[1] = v2;
	return res;
    }
    protected String getLangPaths() {
	Enumeration en = hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:q:SELECT path FROM settings WHERE language='" + language + "'").elements();
	return (String)en.nextElement();
    }
    protected String getLangName() {
	Vector v = hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:q:SELECT name FROM settings WHERE language='" + language + "'");
	return (String)v.firstElement();
    }
    public Vector getLanguages() {
	Enumeration en = hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:q:SELECT language FROM settings").elements();
	Vector v = new Vector();
	while (en.hasMoreElements()) v.addElement(en.nextElement());
	return v;
    }
    protected Vector[] getGames() {
	Enumeration en = hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:q:SELECT language, games FROM settings").elements();

	Vector v1 = new Vector();
	Vector v2 = new Vector();
	
	while (en.hasMoreElements()) {
	    String s = (String)en.nextElement();
	    String lng = s.substring(0, s.indexOf(" & "));
	    String val = s.substring(s.indexOf(" & ")+3);
	    int ix = val.indexOf(":") + 1;
	    int jx = val.indexOf(":", ix);
	    while (jx != -1) {
		s = val.substring(ix, jx);
		if (lng.equals(language) && !v2.contains(s)) v2.addElement(s);
		if (!v1.contains(s)) v1.addElement(s);
		ix = jx + 1;
		jx = val.indexOf(":", ix);
	    }
	}
	Vector res[] = new Vector[2];
	res[0] = v1;
	res[1] = v2;
	return res;
    }
    protected int getIndex(Object os[], Object o) {
	int ix = -1;
	int i = 0;
	while (i < os.length && ix == -1) {
	    if (os[i].equals(o)) ix = i;
	    i++;
	}
	return ix;
    }
    protected boolean contains(Object[] os, Object o) {
	return getIndex(os, o) != -1;
    }
    /** Updates the sql-database entries according to the dirty-flags (the data that has been
     * changed). */
    protected void updateLanguage() {
	/*
	System.out.println("dirty flags: d_gms = " + d_gms + ", d_wcs = " + d_wcs + 
			   ", d_fcs = " + d_fcs + ", d_sdb = " + d_sdb + ", d_tdb = " + d_tdb);
	*/

	// If the language is new, it has to be added all the appropriate places
	if (d_lng) {
	    // Create the entries in the settingsdb
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:INSERT INTO settings (language) values ('" + language + "')");
	    //hc.sendQuery(hc.SQL_COMMAND+"2:u:INSERT INTO functions VALUES('" + language + "',"+
	    //	 "'','')");
	    //hc.sendQuery(hc.SQL_COMMAND+"2:u:INSERT INTO games VALUES('" + language + "','')");
	    //  hc.sendQuery(hc.SQL_COMMAND+"2:u:INSERT INTO lang_path VALUES('" + language +
	    //		 "', '')");
	    //  hc.sendQuery(hc.SQL_COMMAND+"2:u:INSERT INTO wordclasses VALUES('" + language +
	    //		 "', '', '', '', '')");
	}
	if (d_lng || d_gms) {
	    // Create the entries in the highscoredb
	    for (int i = 0; i < gms.length; i++) {
		if (gms[i].getState()) {
		    hc.sendQuery(HitServerInterface.SQL_COMMAND+"1:u:CREATE TABLE IF NOT EXISTS " + 
				 language + "_" + gms[i].getLabel() + 
				 " (date DATETIME, name VARCHAR(12), " +
				 "country CHAR(3), score INT(11), hitrate FLOAT(3,1))");
		} else {
		    hc.sendQuery(HitServerInterface.SQL_COMMAND+"1:u:DROP TABLE IF EXISTS " + 
				 language + "_" + gms[i].getLabel());
		}
	    }
	}
	if (d_gms) {
	    // Insert the games into the database
	    String games = ":";
	    for (int i = 0; i < gms.length; i++) 
		if (gms[i].getState()) games += gms[i].getLabel() + ":";
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:UPDATE settings SET games='" + games +"' " +
			 "WHERE language='"+language+"'");
	}
	if (d_wcs) {
	    // Update the wordclasses
	    String wc_all = ":";
	    String wc_nv = ":";
	    String wc_minor = ":";
	    String wc_major = ":";
	    for (int i = 0; i < wcs.length; i++) {
		if (wcs[i].getState()) {
		    String wc = wcs[i].getLabel();
		    wc_all += wc+":";
		    wc_nv += (wc.equals("n") || wc.equals("v") ? wc + ":" : "");
		    wc_minor += (":art:conj:prp:pron:".indexOf(":"+wc+":") != -1 ? 
				 wc + ":" : "");
		    wc_major += (":adj:adv:n:v:".indexOf(":"+wc+":") != -1 ?
				 wc + ":" : "");
		}
	    }
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:UPDATE settings SET "+
			 "wc_all = '"+wc_all+"', wc_nv = '"+wc_nv+"', wc_minor = '"+wc_minor +
			 "', wc_major = '"+wc_major+"' WHERE language='"+language+"'");
	}
	
	if (d_fcs) {
	    // Update the functions
	    String fs = "|";
	    String fl = "|";
	    for (int i = 0; i < fcs.length; i++) {
		fs += f_short[i].getText()+"|";
		fl += f_long[i].getText()+"|";
	    }
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:UPDATE settings SET func_short='"+fs+"' " +
			 ", func_long = '"+fl+"' WHERE language = '"+language+"'");
	}

	if (d_nam) {
	    // Update the name of the language
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:UPDATE settings SET name='" +
			 name.getText() + "' WHERE language='" +language+ "'");
	}
	if (d_sdb) {
	    // Update the sentencepaths
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:UPDATE settings SET path='" + 
			 lang_path.getText() + "' WHERE language='"+language+"'");
	    // And ask the HitServer to update its representation of the
	    // langpaths
	    hc.sendCommand(HitServerInterface.RELOAD_SETTINGS);
	}
	if (d_enc) {
	    // Update the encoding
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:UPDATE settings SET encoding='" +
			 encoding.getText() + "' WHERE language='" + 
			 language+"'");
	}
	
	// Clear the dirty-flags
	d_nam = false;
	d_lng = false;
	d_sdb = false;
	d_enc = false;
	d_gms = false;
	d_fcs = false;
	d_wcs = false;
	validate();
    }
    protected void addLanguage() {
	if (!changeLanguage()) return;
	if (d_lng) {// There has allready been added one language
	    languages = getLanguages();
	    language = (String)languages.firstElement();
	    d_lng = false;
	}
	AddLanguageDialog ald = new AddLanguageDialog(this);
	if (!isShowing()) show();
	hide();
	ald.pack();
	ald.show();
	String lng = ald.getLanguage();
	ald.dispose();
	if (lng != null && languages.contains(lng)) {
	    MessageDialog md = new MessageDialog(this, "Language allready exists!");
	    md.add("Center", new Label("The language you tried to create does allready exist"));
	    md.pack();
	    md.show();
	    md.dispose();
	    language = lng;
	    layoutLanguages(languages, false);
	    return;
	}
	if (lng != null) {
	    // Set dirty-flag
	    d_lng = true;
	    languages.addElement(lng);
	    language = lng;
	    initLanguageVars();
	    layoutLanguages(languages, false);
	    validate();
	}
    }
    protected void addFunction() {
	AddFunctionDialog afd = new AddFunctionDialog(this);
	hide();
	afd.pack();
	afd.show();
	String s = afd.getShort();
	if (s != null) {
	    // Set dirty flag
	    d_fcs = true;
	    int ix = -1, i = 0;
	    // Find index
	    while (i < fcs.length && ix == -1) {
		if (fcs[i].getState()) ix = i;
		i++;
	    }
	    if (ix == -1) ix = fcs.length;
	    
	    TextField tfs[] = f_short;
	    TextField tfl[] = f_long;
	    Checkbox cbs[] = fcs;
	    f_short = new TextField[tfs.length+1];
	    f_long = new TextField[tfl.length+1];
	    fcs = new Checkbox[cbs.length+1];
	    i = 0;
	    while (i < f_short.length) {
		if (i < ix) {
		    f_short[i] = tfs[i];
		    f_long[i] = tfl[i];
		    fcs[i] = cbs[i];
		} else if (i == ix) {
		    f_short[i] = new TextField(s);
		    f_long[i] = new TextField(afd.getLong());
		    fcs[i] = new Checkbox(null, null, false);
		} else {
		    f_short[i] = tfs[i-1];
		    f_long[i] = tfl[i-1];
		    fcs[i] = cbs[i-1];
		}
		i++;
	    }
	    layoutLanguage();
	}
    }
    protected void removeFunctions() {
	int cnt = 0;
	for (int i = 0; i < fcs.length; i++) {
	    if (fcs[i].getState()) {
		cnt++;
		current.remove(fcs[i]);
		current.remove(f_short[i]);
		current.remove(f_long[i]);
	    } else {
		fcs[i-cnt] = fcs[i];
		f_short[i-cnt] = f_short[i];
		f_long[i-cnt] = f_long[i];
	    }
	}
	if (cnt > 0) {
	    // Set dirty-flag
	    d_fcs = true;
	    Checkbox cbs[] = new Checkbox[fcs.length-cnt];
	    TextField tfs[] = new TextField[f_short.length-cnt];
	    TextField tfl[] = new TextField[f_long.length-cnt];
	    System.arraycopy(fcs, 0, cbs, 0, cbs.length);
	    System.arraycopy(f_short, 0, tfs, 0, tfs.length);
	    System.arraycopy(f_long, 0, tfl, 0, tfl.length);		    
	    fcs = cbs;
	    f_short = tfs;
	    f_long = tfl;
	    current.validate();
	    validate();
	}
    }
    protected void refresh() {
	languages = getLanguages();
	if (!languages.contains(language)) language = (String)languages.firstElement();
	d_lng = false;
	layoutLanguages(languages, true);
    }
    public void validate() {
	if (b_update != null) {
	    if (d_sdb || d_enc || d_gms || d_wcs || d_fcs || d_lng || d_nam) {
		b_update.enable();
	    } else {
		b_update.disable();
	    }
	}
	super.validate();
    }
    protected boolean changeLanguage() {
	boolean res = true;
	if (d_lng || d_sdb || d_enc || d_gms || d_wcs || d_fcs || d_nam) {
	    YesNoDialog md = new YesNoDialog(this, "Language has changed");
	    TextArea ta = new TextArea(3, 60);
	    ta.setText("The language has changed.\n"+
		       "Changing to another language would discard the changes.\n"+
		       "Do you wish to discard the changes?");
	    ta.setEditable(false);
	    md.add("Center", ta);
	    md.pack();
	    md.show();
	    md.hide();
	    if (!md.isOk()) res = false;
	    md.dispose();
	    show();
	}
	return res;
    }
    protected void removeLanguage() {
	YesNoDialog md = 
	    new YesNoDialog(this, "Remove language permanently?",
			    "WARNING: This will completely erase all the high"+
			    " scores\n and settings for this language from"+
			    "the database!!\n\n"+
			    "Are you sure you want to remove the language?");
	if (md.isOk()) {
	    md.dispose();
	    for (int i = 0; i < gms.length; i++) {
		// Try to remove all (also not tagged) games
		doRemoveGame(language, gms[i].getLabel());
	    }
	    // **** SettingsDB
	    // Remove the entries
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:DELETE FROM settings WHERE "+
			 "language='" + language + "'");
	    // **** SentenceDB
	    // Remove the sentencedb-entry
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"0:u:DROP TABLE IF EXISTS "+language);

	    // **** TreeDB
	    // Remove the treedb-entry
	    hc.sendQuery(HitServerInterface.SQL_COMMAND+"0:u:DROP TABLE IF EXISTS "+language);
	    
	    languages = getLanguages();
	    language = (String)languages.firstElement();
	    d_lng = false;
	    layoutLanguages(languages, true);
	} else md.dispose();
    }
    /** This method adds a game to the possible games, it also appends the game
     * to the games supported by this language */
    protected void doAddGame() {
	InputDialog md = new InputDialog(this, "Add game", 
					 "Name of the game:");
	md.pack();
	md.show();
	if (md.isOk()) {
	    String name = md.getInput();
	    if (name == null || name.equals("")) 
		new MessageDialog(this, "Not a valid game!", 
				  "The name of the game can not be an empty "+
				  "string.\n");
	    else {
		// Ok, add the game.
		Checkbox cbs[] = new Checkbox[gms.length+1];
		System.arraycopy(gms, 0, cbs, 0, gms.length);
		cbs[gms.length] = new Checkbox(name, true);
		gms = cbs;
		d_gms = true;
		layoutLanguage();
	    }
	}
	md.dispose();
    }
    protected void doRemoveGames() {
	YesNoDialog md = 
	    new YesNoDialog(this, "Permanently remove game?",
			    "IMPORTANT: If you continue, all the selected\n"+
			    "games and the corresponding high scores will\n"+
			    "be permanently removed from the database.\n"+
			    "\nDo you wish to remove the games?");
	if (md.isOk()) {
	    int cnt = 0, ix = 0, jx = 0;
	    for (int i = 0; i < gms.length; i++) if (gms[i].getState()) cnt++;
	    Checkbox cbs[] = new Checkbox[gms.length - cnt];
	    // Okay, copy all the deselected games into cbs, move all the
	    // selected games to the front of gms.
	    for (int i = 0; i < gms.length; i++) 
		if (!gms[i].getState()) cbs[ix++] = gms[i];
		else gms[jx++] = gms[i];
	    for (int i = 0; i < languages.size(); i++)
		for (int j = 0; j < cnt; j++) 
		    if (gms[j].getState())
			doRemoveGame((String)languages.elementAt(i), 
				     gms[j].getLabel());
	    // Now set gms = cbs;
	    gms = cbs;
	    layoutLanguage();
	}
    }
    protected void doRemoveGame(String lang, String game) {
	// Remove the highscoreentries.
	hc.sendQuery(HitServerInterface.SQL_COMMAND+"1:u:DROP TABLE IF EXISTS "
		     + lang + "_" + game);
	// Update the settings-entry (not for kids!!)
	hc.sendQuery(HitServerInterface.SQL_COMMAND+"2:u:UPDATE settings SET games=CONCAT(LEFT(games, @ix:=LOCATE(':"+game+":', games)),RIGHT(@ix+5, LENGTH(games)-@ix-5)) WHERE language='" + lang + "' AND LOCATE(':"+game+":', games)!=0");
    }
    /** Selects all the games, and updates the d_gms flag accordingly */
    protected void doSelectAllGames() {
	int cnt = 0;
	for (int i = 0; i < gms.length; i++) {
	    if (!gms[i].getState()) cnt++;
	    gms[i].setState(true);
	}
	if (cnt > 0) d_gms = true;
    }
    protected void doDeselectAllGames() {
	int cnt = 0;
	for (int i = 0; i < gms.length; i++) {
	    if (gms[i].getState()) cnt++;
	    gms[i].setState(false);
	}
	if (cnt > 0) d_gms = true;
    }
}

