package visl.rasmus.hitserver.server;

import java.io.DataInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import visl.rasmus.hitserver.client.HitClient;
import visl.rasmus.hitserver.data.HighScoreItem;
import visl.rasmus.hitserver.data.Sentence;
import visl.rasmus.hitserver.data.SentenceDB;
import visl.rasmus.hitserver.data.TreeDB;
import visl.rasmus.hitserver.data.TreeNode;
import visl.rasmus.hitserver.data.Word;
import visl.rasmus.utils.Timer;
import visl.rasmus.utils.TimerListener;

public class HitServer extends Thread implements HitServerInterface, 
						 TimerListener {
    public static final String VERSION = "HitServer v1.3 gamma";
    // 2 times a day ~ every 12th hour
    public static final long BACKUP_TIME = 1000*60*60*12;
    // 12 times a day ~ every 3rd hour
    //public static final long BACKUP_TIME = 1000*60*60*3;
    /** SO_TIMEOUT for 1 minute */
    public static final int SO_TIMEOUT = 60000;
    public static final int SQL_RETRIES = 3;
    protected int port;
    protected ServerSocket listen_socket;
    protected java.sql.Connection conn_sdb, conn_hdb, conn_xdb, conn_tdb;

    protected Vector clients;
    protected Hashtable logs;
    protected Timer timer;
    protected boolean running;
    protected String languages[], lang_path[][];
    protected int errcode;

    public HitServer(int port) {
	logs = new Hashtable();
	// Create the logs...
	//logs.put("xx", new Vector());
	Date date = new Date(System.currentTimeMillis());
	// Create the generic ERROR log.
	try {
	    logs.put("xx", new PrintStream(new 
		FileOutputStream("logs/" + date2String(date) + ".xx.log")));
	} catch (IOException e) {
	    System.out.println("Error creating the 'xx' log file : " + e);
	}
	// Create connections to the databases.
	conn_sdb = null;
	conn_hdb = null;
	conn_xdb = null;
	conn_tdb = null;
	try {
	    Class.forName("org.gjt.mm.mysql.Driver").newInstance();
	    conn_sdb = DriverManager.getConnection("jdbc:mysql://localhost/sentencedb?user=dummy");
	    addLog("xx", "Succesfully connected to MySql - sentencedb");
	    conn_hdb = DriverManager.getConnection("jdbc:mysql://localhost/highscoredb?user=dummy");
	    addLog("xx", "Succesfully connected to MySql - highscoredb");
	    conn_xdb = DriverManager.getConnection("jdbc:mysql://localhost/settingsdb?user=dummy");
	    addLog("xx", "Succesfully connected to MySql - settingsdb");
	    conn_tdb = DriverManager.getConnection("jdbc:mysql://localhost/treedb?user=dummy");
	    addLog("xx", "Succesfully connected to MySql - treedb");
	} catch (Exception e) {
	    addLog("xx", "Error while connecting to MySql: " + e);
	}
	updateSettings();
	
	// Setup all the language-specific log-files.
	for (int i = 0; i < languages.length; i++) {
	    try {
		logs.put(languages[i], new PrintStream(new 
		    FileOutputStream("logs/" + date2String(date) + "." +
				     languages[i] + ".log")));
	    } catch (IOException e) {
		System.out.println("Error creating the '" + languages[i] + 
				   "' log file : " + e);
	    }
	}

	clients = new Vector();
	if (port == 0) port = DEFAULT_PORT;
	this.port = port;

	try {
	    listen_socket = new ServerSocket(port);
	} catch (IOException x) {
	    fail(x, "Exception creating server socket");
	}
	addLog("xx", "Server listening on port " + port);
	addLog("xx", "Connecting to MySql!");

	timer = new Timer(BACKUP_TIME);
	timer.addListener(this);
	timer.start();
	// A little hacky, but it is important, that the thread continues.
	this.run();
    }
    public static void fail(Exception e, String s) {
	System.out.println(s + ": " + e);
	System.exit(-1);
    }
    public void run() {
	try {
	    running = true;
	    while (running) {
		Socket client_socket = listen_socket.accept();
		client_socket.setSoTimeout(SO_TIMEOUT);
		Connection c = new Connection(client_socket);
		clients.addElement(c);
	    }
	} catch (IOException e) {
	    System.out.println("Number of connections: " + clients.size());
	    e.printStackTrace();
	    fail(e, "Exception while listening for connections");
	}
	// Close all connections
	synchronized (clients) {
	    while (clients.size() > 0) {
		Connection c = (Connection)clients.firstElement();
		c.close();
	    }
	}
	System.exit(errcode);
    }
    protected void quit(int errcode) {
	try {
	    if (conn_sdb != null) conn_sdb.close();
	    if (conn_hdb != null) conn_hdb.close();
	    if (conn_xdb != null) conn_xdb.close();
	    if (conn_tdb != null) conn_tdb.close();
	    // Close the log-files
	    for (int i = 0; i < languages.length; i++)
		((PrintStream)logs.get(languages[i])).close();
	} catch (Exception e) {}
	this.errcode = errcode;
	if (errcode != 0)
	    addLog("xx", "Server got a RESTART command");
	else
	    addLog("xx", "Server got a SHUTDOWN command");
	running = false;
    }
    public void printEnumeration(PrintStream p, Enumeration en) {
	for ( ; en.hasMoreElements() ; ) {
	    p.println(en.nextElement());
	}
    }
    public void addLog(String language, String s) {
	PrintStream ps = (PrintStream)logs.get(language);
	ps.println(date2String(new Date())+" "+s);
    }
    public void tick() {
	// Wait with this update until the updatings are consistent!!!
	updateSettings();
	reloadSentences();
	reloadTrees();
    }
    private java.sql.Connection openConnection(java.sql.Connection con) 
	throws java.sql.SQLException {
	if (conn_sdb == con) {
	    conn_sdb = DriverManager.getConnection("jdbc:mysql://localhost/sentencedb?user=dummy");
	    con = conn_sdb;
	} else if (conn_hdb == con) {
	    conn_hdb = DriverManager.getConnection("jdbc:mysql://localhost/highscoredb?user=dummy");
	    con = conn_hdb;
	} else if (conn_xdb == con) {
	    conn_xdb = DriverManager.getConnection("jdbc:mysql://localhost/settingsdb?user=dummy");
	    con = conn_xdb;
	} else if (conn_tdb == con) {
	    conn_tdb = DriverManager.getConnection("jdbc:mysql://localhost/treedb?user=dummy");
	    con = conn_tdb;
	}
	return con;
    }
    private int updateSQL2(String sqlstatement, java.sql.Connection con) 
	throws SQLException {
	Statement stmnt = con.createStatement();
	return stmnt.executeUpdate(sqlstatement);
    }
    private int updateSQL(String sqlstatement, java.sql.Connection con) {
	int retries = 0;
	boolean error = false;
	int res = -1;
	do {
	    try {
		if (retries != 0) con = openConnection(con);
		res = updateSQL2(sqlstatement, con);
		error = false;
	    } catch (SQLException x) {
		addLog("xx", "Error updating SQL Database: " + x);
		error = true;
		retries++;
	    }
	} while (error && retries < SQL_RETRIES);
	return res;
    }
    private ResultSet querySQL(String sqlstatement, java.sql.Connection con) {
	int retries = 0;
	boolean error = false;
	ResultSet rS = null;
	do {
	    try {
		if (retries != 0) con = openConnection(con);
		Statement stmnt = con.createStatement();
		rS = stmnt.executeQuery(sqlstatement);
		error = false;
	    } catch (SQLException x) {
		addLog("xx", "Error querying SQL Database: " + x);
		error = true;
		retries++;
	    }
	} while (error && retries < SQL_RETRIES);
	return rS;
    }
    protected ResultSet querySQL2(String sqlstatement, 
				  java.sql.Connection con) 
	throws SQLException {
	ResultSet rS = null;
	Statement stmnt = con.createStatement();
	rS = stmnt.executeQuery(sqlstatement);
	return rS;
    }
    
    protected void lockTable(String table, java.sql.Connection con) {
	// NOTE: This has to be taken care of!!!
	//updateSQL("LOCK TABLES " + table + " WRITE", con);
    }
    protected void unlockTable(String table, java.sql.Connection con) {
	// NOTE: This has to be taken care of!!!
	//updateSQL("UNLOCK TABLES", con);
    }
    protected void reloadSentences() {
	for (int i = 0; i < languages.length; i++) {
	    reloadSentence(i);
	}
    }
    protected void reloadSentence(int i) {
	SentenceDB sdb = new SentenceDB();
	String encoding = "ISO8859_1";
	// Read Sentences into memory
	try {
	    ResultSet rs = querySQL2("SELECT encoding FROM settings WHERE language=\""+ languages[i]+"\"", conn_xdb);
	    if (rs.next())
		encoding = rs.getString(1);
	} catch (Exception e) {
	    addLog("xx", "Error retreiving encoding for " + languages[i] +
		   ": " + e);
	}
	// This disables the encoding...
	encoding = null;
	for (int j = 0; j < lang_path[i].length; j++) {
	    sdb.readSentences(lang_path[i][j], (PrintStream)logs.get("xx"),
			      encoding);
	}
	// Dump sentences to database
	lockTable(languages[i], conn_sdb);
	updateSQL("DROP TABLE IF EXISTS " + languages[i], conn_sdb);
	updateSQL("CREATE TABLE " + languages[i] + "(id BIGINT "+
		  "PRIMARY KEY AUTO_INCREMENT, length INTEGER, words " +
		  "BLOB, forms BLOB)", conn_sdb);
	for (int j = 0; j < sdb.size(); j++) {
	    Sentence s = sdb.getSentence(j);
	    String res[] = sentence2String(s);
	    try {
		updateSQL2("INSERT INTO " + languages[i] + " VALUES(0," +
			   s.size() + ",\"" + res[0] + "\",\"" + res[1] + 
			   "\")", conn_sdb);
	    } catch (Exception x) {
		addLog("xx", "Error adding " + s + ", to database " + languages[i]);
	    }
	}
	// Remove duplicate/ambigiuous sentences
	try {
	    // Get all the ID's
	    ResultSet rS = querySQL2("SELECT DISTINCT t1.id FROM " + 
				     languages[i] + " AS t1, " + languages[i] +
				     " AS t2 WHERE t1.words = t2.words AND " +
				     "t1.id != t2.id ORDER BY t1.id", conn_sdb);
	    StringBuffer dupes = new StringBuffer("(");
	    while (rS.next()) dupes.append(rS.getInt(1)+",");
	    if (dupes.length() == 1) {
		addLog("xx", "No duplicates in " + languages[i]);
	    } else {
		dupes.setCharAt(dupes.length()-1, ')');
		// Delete all the duplicates
		addLog("xx", "Deleted " + 
		       updateSQL2("DELETE FROM " + languages[i] + 
				  " WHERE id IN " + dupes, conn_sdb) + 
		       " duplicate sentences from " + languages[i]);
	    }
	    addLog("xx", "Deleted " +
		   updateSQL2("DELETE FROM " + languages[i] +
			      " WHERE LOCATE(\"/\", forms) != 0", conn_sdb) +
		   " ambiguous sentences from " + languages[i]);
	} catch (Exception x) {
	    addLog("xx", "Error deleting duplicates from " + languages[i]);
	}
	unlockTable(languages[i], conn_sdb);
    }
    protected void reloadTrees() {
	for (int i = 0; i < languages.length; i++) {
	    reloadTree(i);
	}
    }
    protected void reloadTree(int i) {
	TreeDB tdb = new TreeDB();
	// Read Trees into memory
	for (int j = 0; j < lang_path[i].length; j++) {
	    tdb.readTrees(lang_path[i][j]);
	}
	// Dump trees to database
	lockTable(languages[i], conn_tdb);
	updateSQL("DROP TABLE IF EXISTS " + languages[i], conn_tdb);
	updateSQL("CREATE TABLE " + languages[i] + "(id BIGINT "+
		  "PRIMARY KEY AUTO_INCREMENT, length INTEGER, depth INTEGER,"+
		  "data BLOB)", conn_tdb);
	for (int j = 0; j < tdb.size(); j++) {
	    TreeNode n = tdb.getTree(j);
	    try {
		String s = TreeNode.toString(n);
		// Filter out ",',:,;
		for (int k = 0; k < s.length(); k++) {
		    if (s.charAt(k) == '"' ||
			s.charAt(k) == '\'' ||
			s.charAt(k) == ';' ||
			s.charAt(k) == ':')
			s = s.substring(0,k) + s.substring(k+1);
		}
		updateSQL2("INSERT INTO " + languages[i] + " VALUES(0," +
			   n.leafCount() + "," + n.maxDepth() + 
			   ",\"" + s + "\")",
			   conn_tdb);
	    } catch (Exception x) {
		addLog("xx", "Error adding " + n + ", to tree-database " + languages[i] + ":" + x);
	    }
	} 
	// Remove discontinuous trees
	try {
	    // Delete all the discontinuities
	    addLog("xx", "Deleted " + 
		   updateSQL2("DELETE FROM " + languages[i] + 
			      " WHERE LOCATE(\"|1}\", data) != 0", 
			      conn_tdb) + 
		   " discontinuous trees from " + languages[i]);
	    addLog("xx", "Deleted " +
		   updateSQL2("DELETE FROM " + languages[i] +
			      " WHERE LOCATE(\"/\", data) != 0",
			      conn_tdb) +
		   " ambigouos trees from " + languages[i]);
	} catch (Exception x) {
	    addLog("xx", "Error deleting discontinuities from " + languages[i]+
		   ": " + x);
	}
	unlockTable(languages[i], conn_tdb);
    }
    /** Executes the sglstatement on the designated connection.
     * 0 - SentenceDB, 1 - HighScoreDB, 2 - SettingsDB, 3 - TreeDB */
    protected void doSQLStatement(String s, PrintStream o) {
	addLog("xx", "SQL Statement [" + s + "]");
	try {
	    java.sql.Connection con = null;
	    // Which connection?
	    int ix, jx;
	    ix = s.indexOf(':')+1;
	    jx = s.indexOf(':', ix);
	    if (s.substring(ix, jx).equals("0")) con = conn_sdb;
	    else if (s.substring(ix, jx).equals("1")) con = conn_hdb;
	    else if (s.substring(ix, jx).equals("2")) con = conn_xdb;
	    else con = conn_tdb;
	    // Update or Query?
	    ix = jx + 1;
	    jx = s.indexOf(':', ix);
	    if (s.substring(ix,jx).equals("u")) {
		// Update
		o.println(updateSQL(s.substring(jx+1), con));
	    } else {
		// Query
		//addLog("xx", "Executing Query [" + s + "]");
		ResultSet rS = querySQL(s.substring(jx+1), con);
		// Count number of columns in first row!
		int cnt = 1;
		boolean error = false;
		if (rS.next()) {
		    StringBuffer buff = new StringBuffer();
		    do {
			try {
			    String str = rS.getString(cnt);
			    buff.append(str+" & ");
			    cnt++;
			} catch (SQLException x2) {
			    error = true;
			}
		    } while (!error);
		    buff.setLength(buff.length()-3);
		    o.println(buff.toString());
		    cnt--;
		}
		while (rS.next()) {
		    StringBuffer buff = new StringBuffer();
		    for (int i = 1; i <= cnt; i++) {
			buff.append(rS.getString(i)+" & ");
		    }
		    buff.setLength(buff.length()-3);
		    //addLog("xx", "- [" + buff.toString() + "]");
		    o.println(buff.toString());
		}
	    }
	} catch (Exception x) {
	    addLog("xx", "SQL Exception: " + x);
	}
	o.println(EOT);
    }
    /** Updates the settingsdb, that is: wordclasses and functions */
    protected void updateSettings() {
	Date date = new Date();
	languages = getLanguages();
	for (int i = 0; i < languages.length; i++) {
	    if (!logs.containsKey(languages[i]))
		// A new language has been introduced - create log-entry
		try {
		    logs.put(languages[i], new PrintStream(new 
			FileOutputStream("logs/" + date2String(date) + "." +
					 languages[i] + ".log")));
		} catch (FileNotFoundException _) {
		    _.printStackTrace();
		}
	}
	lang_path = getLangPaths();
    }
    /** Returns an array containing all the languages known in the database */
    protected String[] getLanguages() {
	String ra[] = null;
	try {
	    Vector r = new Vector();
	    ResultSet rs = querySQL("SELECT language FROM settings "+
				    "ORDER BY language", 
				    conn_xdb);
	    while (rs != null && rs.next()) {
		r.addElement(rs.getString(1));
	    }
	    ra = new String[r.size()];
	    for (int i = 0; i < ra.length; i++)
		ra[i] = (String)r.elementAt(i);
	} catch (SQLException x) {
	    x.printStackTrace();
	    addLog("xx", "Exception retreiving the languages: " + x);
	}
	return ra;
    }
    /** Returns an array containing all the paths to the different languages
     **/
    protected String[][] getLangPaths() {
	String ra[][] = null;
	try {
	    Vector r = new Vector();
	    ResultSet rs = querySQL("SELECT path FROM settings ORDER "+
				    "BY language", 
				    conn_xdb);
	    while (rs.next()) {
		String s = rs.getString(1);
		if (s.indexOf(' ') != -1) {
		    Vector v = new Vector();
		    int ix = 0;
		    int jx = s.indexOf(' ', ix);
		    while (jx != -1) {
			v.addElement(s.substring(ix, jx));
			ix = jx + 1;
			jx = s.indexOf(' ', ix);
		    }
		    r.addElement(v);
		} else {
		    Vector v = new Vector();
		    v.addElement(s);
		    r.addElement(v);
		}
	    }
	    ra = new String[r.size()][];
	    for (int i = 0; i < ra.length; i++) {
		Vector v = (Vector)r.elementAt(i);
		ra[i] = new String[v.size()];
		for (int j = 0; j < ra[i].length; j++)
		    ra[i][j] = (String)v.elementAt(j);
	    }
	} catch (SQLException x) {
	    addLog("xx", "Exception retreiving language-paths: " + x);
	}
	return ra;
    }
    public String date2String(Date d) {
	return (""+(d.getYear()+1900)+
		(d.getMonth()+1<10?"0"+(d.getMonth()+1):""+(d.getMonth()+1))+
		(d.getDate() < 10 ? "0"+d.getDate() : ""+d.getDate()) +"+"+
		(d.getHours() < 10 ? "0"+d.getHours() : ""+d.getHours())+":" +
		(d.getMinutes() < 10 ? "0"+d.getMinutes() : ""+d.getMinutes())
		);
    }
    // ***********************************************************************
    // * CLASS CONNECTION
    // ***********************************************************************
    public class Connection extends Thread implements TimerListener {
	public static final int TIME_OUT = 30000;
	protected Socket client;
	protected DataInputStream in;
	protected PrintStream out;
	protected boolean local;
	protected int ticcount;
	protected Timer timer;

	public Connection(Socket client_socket) {
	    client = client_socket;
	    local = (client.getInetAddress().getHostAddress().equals("127.0.0.1"));
	    ticcount = 2;
	    try {
		in = new DataInputStream(client.getInputStream());
		out = new PrintStream(client.getOutputStream());
	    } catch (IOException e) {
		try { client.close(); } catch (IOException x) {}
		addLog("xx", "Exception while getting socket streams: " + e);
		return;
	    }
	    timer = new Timer(500);
	    timer.setTimeOut(TIME_OUT);
	    timer.addListener(this);
	    timer.start();
	    timer.pause();
	    this.setDaemon(true);
	    this.start();
	}
	public void close() {
	    try { in.close(); } catch (Exception _) {}
	    try { out.close(); } catch (Exception _) {}
	    try { client.close(); } catch (Exception _) {}
	    timer.stop();
	    clients.removeElement(this);
	}
	protected void send(int i) {
	    send(""+i);
	}
	protected void send(Object o) {
	    send(o.toString());
	}
	protected void send(String s) {
	    timer.reset();
	    out.println(s);
	    timer.pause();
	}
	public void run() {
	    String line = null;
	    int len;
	    try {
		while (true) {
		    timer.reset();
		    line = in.readLine();
		    timer.pause();
		    // *** Some commands are restricted to local access.
		    if (local && line.startsWith(GET_VERSION)) {
			out.println(VERSION);
			out.println(EOT);
		    } else if (local && line.startsWith("CLEAR_CONN")) {
			while (clients.size() > 0) {
			    ((Connection)clients.firstElement()).close();
			}
		    } else if (local && line.startsWith("RESTART")) 
			shutdownServer(100);
		    else if (local && line.startsWith(GET_LOG))
			sendLog(line);
		    else if (local && line.startsWith(QUIT))
			shutdownServer(0);
		    else if (local && line.startsWith(GET_NUM_CONNECTIONS))
			sendNumberOfConnections();
		    else if (local && line.startsWith(RELOAD_SENTENCES)) {
			int ix = -1;
			String s = line.substring(line.indexOf(':')+1);
			if (s.equals("ALL")) reloadSentences();
			else {
			    for (int i = 0; 
				 i < languages.length && ix == -1; 
				 i++)
				if (languages[i].equals(s)) ix = i;
			    if (ix != -1) reloadSentence(ix);
			}
		    } else if (local && line.startsWith(RELOAD_TREES)) {
			int ix = -1;
			String s = line.substring(line.indexOf(':')+1);
			if (s.equals("ALL")) reloadTrees();
			else {
			    for (int i = 0; 
				 i < languages.length && ix == -1; 
				 i++)
				if (languages[i].equals(s)) ix = i;
			    if (ix != -1) reloadTree(ix);
			}
		    } else if (local && line.startsWith(RELOAD_SETTINGS)) {
			updateSettings();
		    } else if ((local && line.startsWith(SQL_COMMAND)) ||
			       (line.startsWith(SQL_COMMAND) && 
				line.indexOf(":q:") == 5))
			doSQLStatement(line, out);
		    else if (line.startsWith(GET_HISCORE)) 
			sendHighScore(line);
		    else if (line.startsWith(GET_SENTENCES)) 
			sendSentences(line);
		    else if (line.startsWith(GET_SENTENCE))
			sendSentence(line);
		    else if (line.startsWith(GET_TREES))
			sendTrees(line);
		    else if (line.startsWith(GET_WCS))
			sendWordClasses(line);
		    else if (line.startsWith(GET_FUNCS))
			sendFunctions(line);
		    else if (line.startsWith(PUT_HISCORE)) 
			receiveHighScore(line);
		    else if (line.startsWith(QUE_HISCORE))
			queryHighScore(line);
		    else if (line.startsWith(DISCONNECT)) {
			close();
			break;
		    } else {
			addLog("xx", client.getInetAddress()+":"+
			       client.getPort()+
			       ":Unrecognized command from client["+line+"]");
		    }
		}
	    } catch (Exception e) {
		addLog("xx", client.getInetAddress()+":"+client.getPort()+
		       ":Exception while communicating with client ["+line+
		       "]: "+e);
	    } finally {
		close();
		//try { client.close(); } catch (IOException x) {}
	    }
	}
	/** If this tick is called more than once, 
	 * the connection has timed-out. */
	public void tick() {
	    addLog("xx", client.getInetAddress()+":"+client.getPort()+
		   ":Connection timed-out");
	    close();
	    this.interrupt();
	}
	private void sendSentences(String line) throws Exception {
	    try {
		// "GetSentences:<language>;<game>;<form>;<min>;<max>;<num>"
		String rs[] = extractStrings(line);
		String lng = rs[0];
		addLog(lng, client.getInetAddress()+":"+client.getPort()+":"+
		       line);
		String game = rs[1];
		int ix = 0, jx = rs[2].indexOf(';');
		String form = rs[2].substring(ix, jx);
		ix = jx + 1;
		jx = rs[2].indexOf(";", ix);
		int min = Integer.parseInt(rs[2].substring(ix, jx));
		ix = jx + 1;
		jx = rs[2].indexOf(";", ix);
		int max = Integer.parseInt(rs[2].substring(ix, jx));
		ix = jx + 1;
		int num = Integer.parseInt(rs[2].substring(ix));
		//SentenceDB sentencedb = (SentenceDB)sentencedbs.get(lng);

		ResultSet rS = null;
		StringBuffer buff = new StringBuffer();
		if (form.indexOf(':') == -1) {
		    buff.append("SELECT DISTINCT words,forms FROM " + lng +
				" WHERE length >= " + min + " AND length <= " +
				max +
				" AND !(LOCATE(\":"+form+":\",forms)=0) " +
				"ORDER BY RAND() LIMIT "+num);
		} else {
		    buff.append("SELECT DISTINCT words,forms FROM " + lng +
				" WHERE length >= " + min + " AND length <= " +
				max + " AND (");
		    ix = form.indexOf(':')+1;
		    jx = form.indexOf(':',ix);
		    while (jx != -1) {
			buff.append("!(LOCATE(\":"+form.substring(ix,jx)+":\""+
				    ", forms) = 0) OR ");
			ix = jx + 1;
			jx = form.indexOf(':', ix);
		    }
		    buff.setLength(buff.length()-3);
		    buff.append(") ORDER BY RAND() LIMIT " + num);
		}
		rS = querySQL(buff.toString(), conn_sdb);
		Sentence s = new Sentence();
		while (rS.next()) {
		    String sr = string2SentenceStr(rS.getString(1), 
						   rS.getString(2));
		    addLog(lng, client.getInetAddress()+":"+client.getPort()+
			   ":"+sr);
		    send(sr);
		}
		send(EOT);
	    } catch (Exception x) {
		addLog("xx", client.getInetAddress()+":"+client.
		       getPort()+":Exception while sending sentences: " + x);
		throw x;
	    }
	}
	private void sendTrees(String line) throws Exception {
	    try {
		// "GetTrees:<language>;<game>;<min>;<max>;<num>"
		String rs[] = extractStrings(line);
		String lng = rs[0];
		addLog(lng, client.getInetAddress()+":"+client.getPort()+":"+
		       line);
		String game = rs[1];
		int ix = 0, jx = rs[2].indexOf(';');
		int min = Integer.parseInt(rs[2].substring(ix, jx));
		ix = jx + 1;
		jx = rs[2].indexOf(";", ix);
		int max = Integer.parseInt(rs[2].substring(ix, jx));
		ix = jx + 1;
		int num = Integer.parseInt(rs[2].substring(ix));
		//SentenceDB sentencedb = (SentenceDB)sentencedbs.get(lng);

		ResultSet rS = null;
		StringBuffer buff = new StringBuffer();
		if (":ar:ru:".indexOf(lng) != -1)
		    buff.append("SELECT DISTINCT data FROM " + lng +
				" WHERE length >= " + min + " AND length <= " +
				max +
				" ORDER BY RAND() LIMIT "+num);
		else
		    buff.append("SELECT DISTINCT data FROM " + lng +
				" WHERE length >= " + min + " AND length <= " +
				max + " AND LOCATE(\"{cl|\", data) = 6 "+
				"ORDER BY RAND() LIMIT "+num);
		
		rS = querySQL(buff.toString(), conn_tdb);
		while (rS.next()) {
		    send(rS.getString(1));
		}
		send(EOT);
	    } catch (Exception x) {
		addLog("xx", client.getInetAddress()+":"+client.
		       getPort()+":Exception while sending trees: " + x);
		throw x;
	    }
	}
	private void sendSentence(String line) throws Exception {
	    try {
		// "GetSentence:<language>;<game>;<num>[;min;max]"
		String rs[] = extractStrings(line);
		String lng = rs[0];
		addLog(lng, client.getInetAddress()+":"+client.getPort()+":"+
		       line);
		String game = rs[1];
		int num = -2, max = -1;
		if (rs[2].indexOf(';') == -1)
		    num = Integer.parseInt(rs[2]);
		else {
		    num = Integer.
			parseInt(rs[2].substring(0,rs[2].indexOf(';')));
		    max = Integer.parseInt(rs[2].
					   substring(rs[2].indexOf(';')+1));
		}
		//SentenceDB sentencedb = (SentenceDB)sentencedbs.get(lng);
		String comm = "SELECT DISTINCT words,forms FROM " + lng;
		if (num == -1)
		    // Random sentence
		    comm += " ORDER BY RAND() LIMIT 1";
		else if (max == -1)
		    // Specific sentence
		    comm += " WHERE id=" + num;
		else if (num >= 0 && max >= num)
		    // Specific sentence with number of words
		    comm += " WHERE length >= " + num + 
			" AND length <= " + max + " ORDER BY RAND() LIMIT 1";
		ResultSet rS = querySQL(comm, conn_sdb);
		// Only send a sentence if at least one match was found!
		if (rS.next()) {
		    send(string2SentenceStr(rS.getString(1),
					     rS.getString(2)));
		} else {
		    send("null");
		}
		send(EOT);
	    } catch (Exception x) {
		addLog("xx", client.getInetAddress()+":"+client.getPort()+
		       ":Exception while sending sentence: " + x);
		throw x;
	    }
	}
	private void sendNumberOfConnections() throws Exception {
	    try {
		send(clients.size());
		send(EOT);
	    } catch (Exception x) {
		addLog("xx", client.getInetAddress()+":"+client.getPort()+
		       ":Exception while sending # connections: " + x);
		throw x;
	    }
	}
	private void sendHighScore(String s) throws Exception {
	    // Format: "GetHiScore:<language>;<game>;<type>
	    String rs[] = extractStrings(s);
	    String lng = rs[0];
	    addLog(lng, client.getInetAddress()+":"+client.getPort()+":"+s);
	    String game = rs[1];
	    String st = rs[2];

	    long seconds = 0;
	    int num = 10;
	    String comm = "SELECT * FROM " + lng + "_" + game + " ";
	    if (st.equals(T100Y)) {
		comm += "WHERE NOW() - INTERVAL 1 YEAR <= date ";
		num = 100;
	    } else if (st.equals(T100M)) {
		comm += "WHERE NOW() - INTERVAL 1 MONTH <= date ";
		num = 100;
	    } else if (st.equals(T10Y)) {
		comm += "WHERE NOW() - INTERVAL 1 YEAR <= date ";
	    } else if (st.equals(T10M)) {
		comm += "WHERE NOW() - INTERVAL 1 MONTH <= date ";
	    } else if (st.equals(T10W)) {
		comm += "WHERE NOW() - INTERVAL 7 DAY <= date ";
	    } else if (st.equals(T10D)) {
		comm += "WHERE NOW() - INTERVAL 1 DAY <= date ";
	    } else if (st.equals(TALL)) {
		num = -1;
	    }
	    comm += "ORDER BY score*100 + hitrate DESC ";
	    if (num != -1)
		comm += "LIMIT " + num;
	    ResultSet rS = querySQL(comm, conn_hdb);
	    while (rS.next()) {
		HighScoreItem hsi = new HighScoreItem(rS.getString(2),
						      rS.getString(3),
						      rS.getInt(4),
						      null,
						      rS.getFloat(5));
		String dstr = rS.getString(1);
		// Format: YYYY-MM-DD HH:MM:SS
		//         0123 56 89 11 11 11
		//                    12 45 78
		// The ResultSet should be able to extract an java.sql.Date,
		// that specializes java.util.Date, but it doesn't seem to work
		hsi.date = new Date(Integer.parseInt(dstr.substring(0,4))-1900,
				    Integer.parseInt(dstr.substring(5,7))-1,
				    Integer.parseInt(dstr.substring(8,10)),
				    Integer.parseInt(dstr.substring(11,13)),
				    Integer.parseInt(dstr.substring(14,16))
				    );
		send(hsi);
	    }
	    send(EOT);
	}
	private void receiveHighScore(String s) throws Exception {
	    // Format: PutHiScore:<language>;<game>;<data>
	    try {
		String rs[] = extractStrings(s);
		String lng = rs[0];
		addLog(lng, client.getInetAddress()+":"+client.getPort()+":"+
		       s);
		String game = rs[1];
		HighScoreItem hsi = new HighScoreItem(rs[2]);
		// Make sure the time is measured in server time.
		String comm = "INSERT INTO " + lng + "_" + game + " VALUES("+
		    "NOW(), \"" + hsi.name + "\", \"" + hsi.country + "\", " +
		    hsi.score + ", " + hsi.hitrate + ")";
		updateSQL(comm, conn_hdb);
	    } catch (Exception x) {
		addLog("xx", client.getInetAddress()+":"+client.getPort()+
		       ":Exception while receiving highscore ["+s+"]:"+x);
		throw x;
	    }
	}
	/** Queries the database for the position in the highscore */
	private void queryHighScore(String s) throws Exception {
	    try {
		String rs[] = extractStrings(s);
		String lng = rs[0];
		addLog(lng, client.getInetAddress()+":"+client.getPort()+":"+
		       s);
		String game = rs[1];
		int hashcode = Integer.parseInt(rs[2]);
		ResultSet rS = querySQL("SELECT COUNT(*) FROM " +
					lng + "_" + game + 
					" WHERE score*100+hitrate "+
					">= " + hashcode, conn_hdb);
		int r = -1;
		if (rS.next()) {
		    r = rS.getInt(1);
		}
		send(r);
		send(EOT);
	    } catch (Exception e) {
		addLog("xx", client.getInetAddress()+":"+client.getPort()+
		       ":Exception while quering the highscore ["+s+"]:"+e);
		throw e;
	    }
	}
	private void sendLog(String s) throws Exception {
	    try {
		/* Disabled for now
		int ix = s.indexOf(':')+1;
		String lng = s.substring(ix);
		Vector v = (Vector)logs.get(lng);
		printEnumeration(out, v.elements());
		*/
		send(EOT);
	    } catch (Exception x) {
		addLog("xx", client.getInetAddress()+":"+client.getPort()+
		       ":Exception while sending log: " + x);
		throw x;
	    } 
	}
	/** Converts the string <CODE>{COMMAND}:{language};{game};{opts} to
	 * an array of strings with 3 elements (omitting the COMMAND)*/
	private String[] extractStrings(String s) {
	    String res[] = new String[3];
	    int ix = s.indexOf(":") + 1;
	    int jx = s.indexOf(";", ix);
	    res[0] = s.substring(ix, jx);
	    ix = jx + 1;
	    jx = s.indexOf(";", ix);
	    res[1] = s.substring(ix, jx);
	    ix = jx + 1;
	    res[2] = s.substring(ix);
	    return res;
	}
	private void shutdownServer(int errcode) {
	    timer.stop();
	    quit(errcode);
	}
	protected void sendWordClasses(String s) {
	    try {
		int ix = s.indexOf(':')+1;
		int jx = s.indexOf(';',ix);
		String lang = s.substring(ix,jx);
		String col = s.substring(jx+1);
		addLog(lang, client.getInetAddress()+":"+client.getPort()+":sendWordClasses["+s+"]");
		ResultSet rS = querySQL("SELECT wc_"+col+" FROM settings "+
					"WHERE language = \"" + lang + "\"", 
					conn_xdb);
		if (rS.next()) {
		    addLog(lang, client.getInetAddress()+":"+client.getPort()+
			   ":Sending result back!");
		    send(rS.getString(1));
		} else {
		    addLog(lang, client.getInetAddress()+":"+client.getPort()+
			   ":No result found!");
		    send("null");
		}
		send(EOT);
	    } catch (Exception e) {
		addLog("xx", client.getInetAddress()+":"+client.
				   getPort()+":sendWordClasses("+e+")");
	    }
	}
	protected void sendFunctions(String s) {
	    try {
		int ix = s.indexOf(':')+1;
		int jx = s.indexOf(';',ix);
		String lang = s.substring(ix,jx);
		String col = s.substring(jx+1);
		addLog(lang, client.getInetAddress()+":"+client.getPort()+
		       ":sendFunctions["+s+"]");
		ResultSet rS = querySQL("SELECT func_"+col+" FROM settings "+
					"WHERE language = \"" + lang + "\"", 
					conn_xdb);
		if (rS.next()) {
		    addLog(lang, client.getInetAddress()+":"+client.getPort()+
			   ":Sending result back:"+rS.getString(1));
		    send(rS.getString(1));
		} else {
		    addLog(lang, client.getInetAddress()+":"+client.getPort()+
			   ":No result found!");
		    send("null");
		}
		send(EOT);
	    } catch (Exception e) {
		addLog("xx", client.getInetAddress()+":"+client.getPort()+
		       ":sendFunctions("+e+")");
	    }
	}
    }
    /** Converts a : seperated string into a stirng-representation of a 
     * sentence 
     * @see visl.rasmus.hitserver.data.Sentence 
     * Note: words with zero-forms are filtered out of the sentence */
    static private String string2SentenceStr(String ws, String fs) {
	// A little HACKY!!
	// Should use Sentence to find string...
	StringBuffer buff = new StringBuffer("Sentence[");
	int ix1 = Sentence.getNextIndex(ws, ':', 0)+1,
	    ix2 = Sentence.getNextIndex(ws, ':', ix1),
	    jx1 = Sentence.getNextIndex(fs, ':', 0)+1,
	    jx2 = Sentence.getNextIndex(fs, ':',jx1);
	try {
	    while (ix2 != -1) {
		String form = fs.substring(jx1, jx2);
		if (!form.equals("0"))
		    buff.append(ws.substring(ix1,ix2)+":"+
				form+";");
		ix1 = ix2+1;
		jx1 = jx2+1;
		ix2 = Sentence.getNextIndex(ws, ':',ix1);
		jx2 = Sentence.getNextIndex(fs, ':',jx1);
	    }
	    buff.setCharAt(buff.length()-1,']');
	} catch (RuntimeException e) {
	    throw new RuntimeException("[string2SentenceStr("+ws+","+fs+")]"+
				       e.getMessage());
	}
	return buff.toString();
    }
    /** Converts a Sentence into two : seperated strings containing the words
     * forms.
     * NB: filters out any illegal characters */
    static private String[] sentence2String(Sentence s) {
	String res[] = new String[2];
	StringBuffer ws = new StringBuffer(":");
	StringBuffer fs = new StringBuffer(":");
	for (int i = 0; i < s.size(); i++) {
	    Word w = s.getWord(i);
	    ws.append(Sentence.escape(w.text) + ":");
	    fs.append(Sentence.escape(w.form) + ":");
	}
	int ix = -1;
	// Filter words
	while ((ix = ws.toString().indexOf('"',ix+1)) != -1) {
	    ws.insert(ix++, '\\');
	}
	ix = -1;
	// Filter forms
	while ((ix = fs.toString().indexOf('"',ix+1)) != -1) {
	    fs.insert(ix++, '\\');
	}
	
	res[0] = ws.toString();
	res[1] = fs.toString();
	return res;
    }
    /** Main routine - start-up the server (if it isn't running). */
    public static void main(String args[]) {
	int port = DEFAULT_PORT, cnt = 0;
	HitClient hc = new HitClient("localhost", port, null, null);
	boolean err = true;
	while (err && cnt < 3) {
	    try {
		hc.connect();
		hc.close();
		err = true;
	    } catch (Exception e) {
		err = false;
	    }
	    if (err) {
		System.out.println("HitServer is allready running or port in use!");
		System.out.print("Retrying in 10 secs.");
		for (int i = 0; i < 10; i++) {
		    try { Thread.sleep(1000); } 
		    catch (InterruptedException _) {}
		    System.out.print(".");
		}
		System.out.println("time's up!");
	    }
	    cnt++;
	}
	if (!err) {
	    System.out.println("port is free, starting server!");
	    new HitServer(port);
	}
    }
}

    
    
    
    
