package visl.rasmus.hitserver.server;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Vector;

import visl.rasmus.hitserver.client.HitXtraClient;
import visl.rasmus.hitserver.data.TreeNode;

/** This class is a frontend to the HitServer class 
 * @author Rasmus Lock Larsen, VISL March 2001 */
public class HitServerFrontEnd implements HitServerInterface {
    protected int language, game;
    protected HitXtraClient hc;
    public HitServerFrontEnd(int port) {
	language = 0;
	game = 0;
	hc = new HitXtraClient("localhost", port, 
			       LANGUAGES[0], GAMES[0]);
    }
    public void run() {
	new KeyboardListener();
    }
    public void printEnumeration(PrintStream p, Enumeration en) {
	for ( ; en.hasMoreElements() ; ) {
	    p.println(en.nextElement());
	}
    }
    public void printVectorReverse(PrintStream p, Vector v) {
	for (int i = v.size()-1; i >= 0; i--)
	    p.println(v.elementAt(i));
    }
    public void printVector(PrintStream p, Vector v) {
	printEnumeration(p, v.elements());
    }
    public void doTestFf() {
	Enumeration en = 
	    hc.sendQuery(SQL_COMMAND+"3:q:"+
			 "SELECT * FROM " + 
			 LANGUAGES[language]).elements();
	TreeNode nn = new TreeNode();
	Vector forms = new Vector();
	Vector functions = new Vector();
	while (en.hasMoreElements()) {
	    TreeNode n = TreeNode.buildTree(nn.getClass().getName()+"["+((String)en.nextElement())+"]"); 
	    Vector v = n.getFormFunction();
	    for (int i = 0; i < v.size(); i++) {
		String s = (String)v.elementAt(i);
		if (s.startsWith("f:")) {
		    s = s.substring(2);
		    if (!forms.contains(s)) forms.addElement(s);
		} else {
		    s = s.substring(2);
		    if (!functions.contains(s)) functions.addElement(s);
		}
	    }
	}
	System.out.println("Forms:");
	for (int i = 0; i < forms.size(); i++) {
	    System.out.println(" "+forms.elementAt(i));
	}
	System.out.println("Functions:");
	for (int i = 0; i < functions.size(); i++) {
	    System.out.println(" "+functions.elementAt(i));
	}
    }
    // ***********************************************************************
    // * CLASS KEYBOARDLISTENER
    // ***********************************************************************
    public class KeyboardListener extends Thread {
	
	public KeyboardListener() {
	    this.start();
	}
	public void run() {
	    try {
		DataInputStream keyin = new DataInputStream(System.in);
		int state = 0;
		printMenu(state);
		while (true) {
		    // Check for user input
		    String s = keyin.readLine();
		    // Global keys
		    if (s.equals("l")) {
			if (language < LANGUAGES.length-1) language++;
			else language = 0;
			hc.setLanguage(LANGUAGES[language]);
		    } else if (s.equals("g")) {
			if (game < GAMES.length-1) game++;
			else game = 0;
			hc.setGame(GAMES[game]);
		    } else {
			switch (state) {
			case 0: // Main Menu
			    if (s.equals("h")) ;
			    else if (s.equals("s")) state = 2;
			    else if (s.equals("hi")) state = 3;
			    else if (s.equals("a")) state = 4;
			    else if (s.equals("f")) state = 5;
			    else if (s.equals("t")) state = 6;
			    else if (s.startsWith("q")) System.exit(0);
			    else {
				System.out.println("Unrecognized command!");
			    }
			    break;
			case 2: // Sentence Menu
			    if (s.equals("r")) {
				System.out.println(hc.getRandomSentence());
			    } else if (s.startsWith("r ")) {
				try {
				    int ix = s.indexOf(" ", 2);
				    String form = s.substring(2,ix);
				    ix = ix+1;
				    int jx = s.indexOf(" ", ix);
				    int min = Integer.parseInt(s.substring(ix,jx));
				    ix = jx + 1;
				    int max = Integer.parseInt(s.substring(ix));
				    System.out.println(hc.getRandomSentencesWithForm(form, min, max, 1).elementAt(0));
				} catch (NumberFormatException x) {
				    System.out.println("Unrecognized command : "+x);
				}
			    } else if (s.startsWith("s ")) {
				try {
				    int num = Integer.parseInt(s.substring(2));
				    System.out.println(hc.getSentence(num));
				} catch (NumberFormatException x) {
				    System.out.println("Unrecognized command!");
				}
			    } else if (s.startsWith("sql:")) {
				printVector(System.out,
					    hc.sendQuery(SQL_COMMAND+
							 "0:q:"+
							 s.substring(4)));
			    } else if (s.equals("rl")) {
				hc.sendCommand(RELOAD_SENTENCES+LANGUAGES[language]);
			    } else if (s.equals("rt")) {
				hc.sendCommand(RELOAD_TREES+LANGUAGES[language]);
			    } else if (s.equals("q")) state = 0;
			    else {
				System.out.println("Unrecognized command!");
			    }
			    break;
			case 3: // High Score Menu
			    if (s.equals("y10")) 
				printVector(System.out, hc.getHighScore(T10Y));
			    else if (s.equals("m10"))
				printVector(System.out, hc.getHighScore(T10M));
			    else if (s.equals("w10"))
				printVector(System.out, hc.getHighScore(T10W));
			    else if (s.equals("d10"))
				printVector(System.out, hc.getHighScore(T10D));
			    else if (s.equals("y100"))
				printVector(System.out, hc.getHighScore(T100Y));
			    else if (s.equals("m100"))
				printVector(System.out, hc.getHighScore(T100M));
			    else if (s.equals("a"))
				printVector(System.out, hc.getHighScore(TALL));
			    else if (s.equals("q")) {
				state = 0;
			    } else if (s.startsWith("sql:")) {
				printVector(System.out,
					    hc.sendQuery(SQL_COMMAND+
							 "1:q:"+
							 s.substring(4)));
			    } else {
				System.out.println("Unrecognized command!");
			    }
			    break;
			case 4: // Advanced Menu
			    if (s.equals("log")) {
				printVectorReverse(System.out, hc.getLog());
			    } else if (s.startsWith("c:")) {
				hc.sendCommand(s.substring(2));
			    } else if (s.equals("shutdown")) {
				hc.sendCommand(QUIT);
				System.exit(0);
			    } else if (s.equals("x")) {
				hc.setLanguage("xx");
				printVectorReverse(System.out, hc.getLog());
				hc.setLanguage(LANGUAGES[language]);
			    } else if (s.equals("n")) {
				System.out.println("Number of connections: " + 
						   hc.getNumberOfConnections());
			    } else if (s.equals("q")) {
				state = 0;
			    } else {
				System.out.println("Unrecognized command!");
			    }
			    break;
			case 5:
			    if (s.equals("ra")) {
				hc.sendCommand(RELOAD_SENTENCES+"ALL");
			    } else if (s.equals("rl")) {
				hc.sendCommand(RELOAD_SENTENCES+LANGUAGES[language]);
			    } else if (s.equals("rt")) {
				hc.sendCommand(RELOAD_TREES+LANGUAGES[language]);
			    } else if (s.equals("rat")) {
				hc.sendCommand(RELOAD_TREES+"ALL");
			    } else if (s.equals("rs")) {
				hc.sendCommand(RELOAD_SETTINGS);
			    } else if (s.equals("q")) {
				state = 0;
			    } else {
				System.out.println("Unrecognized command!");
			    }
			    break;
			case 6:
			    if (s.equals("f")) doTestFf();
			    else if (s.equals("q")) state = 0;
			    break;
			}
		    }
		    printMenu(state);
		}
	    } catch (Exception e) {
		System.out.println("Exception in KeyboardListener: "+e);
		e.printStackTrace();
	    }
	}
	private void printMenu(int ix) {
	    System.out.println("");
	    switch (ix) {
	    case 0:
		System.out.println("+---== HitServer v0.4 - MAIN ==---");
		System.out.println("| h        - Help");
		System.out.println("| f        - Files menu");
		System.out.println("| s        - Sentence database menu");
		System.out.println("| hi       - High score database menu");
		System.out.println("| a        - Advanced Options");
		System.out.println("| t        - Test Menu");
		System.out.println("| l        - Change language:"+LANGUAGES[language]);
		System.out.println("| g        - Change game:"+GAMES[game]);
		System.out.println("| q        - Exit");
		break;
	    case 1:
		break;
	    case 2:
		System.out.println("+-----=========== SENTENCES ===========-----");
		System.out.println("+-- HitClient Interface");
		System.out.println("| r <form> <min> <max> - Random Sentence");
		System.out.println("| s <num>              - Specific Sentence");
		System.out.println("+-- Database Interface");
		System.out.println("| sql:<SQL STATEMENT>  - Query the database");
		System.out.println("| rl                   - Reload current language-database");
		System.out.println("| rt                   - Reload current tree-database");
		System.out.println("| l                    - Change language:"+LANGUAGES[language]);
		System.out.println("|");
		System.out.println("| q                    - Back to main menu");
		break;
	    case 3:
		System.out.println("+--===== HIGHSCORE =====--");
		System.out.println("+-- HitClient Interface");
		System.out.println("| y10  - Top10 (Year)");
		System.out.println("| m10  - Top10 (Month)");
		System.out.println("| w10  - Top10 (Week)");
		System.out.println("| d10  - Top10 (Day)");
		System.out.println("| y100 - Top100 (Year)");
		System.out.println("| m100 - Top100 (Month)");
		System.out.println("| a    - All");
		System.out.println("|");
		System.out.println("| l    - Change language:"+LANGUAGES[language]);
		System.out.println("| g    - Change game:"+GAMES[game]);

		System.out.println("+-- SQL Interface");
		System.out.println("| sql:<SQL STATEMENT> - Query the database");
		System.out.println("|");
		System.out.println("| q    - Back to main menu");
		break;
	    case 4:
		System.out.println("+--======== ADVANCED =========--");
		System.out.println("+-- Logs");
		System.out.println("| log          - print log");
		System.out.println("| x            - print xx log");
		System.out.println("| l            - Change language:"+LANGUAGES[language]);
		System.out.println("+-- Network");
		System.out.println("| n            - # of Connections");
		System.out.println("| c:<command>  - Send command directly to server");
		System.out.println("| shutdown     - Shutdown server"); 
		System.out.println("|");
		System.out.println("| q            - Back to main menu");
		break;
	    case 5:
		System.out.println("+--======== FILES ==========--");
		System.out.println("| ra    - Reload all language-databases");
		System.out.println("| rl    - Reload current language-database");
		System.out.println("|");
		System.out.println("| rat   - Reload all tree-databases");
		System.out.println("| rt    - Reload current tree-database");
		System.out.println("|");
		System.out.println("| rs    - Reload the settings");
		System.out.println("| l     - Change language:"+LANGUAGES[language]);
		System.out.println("|");
		System.out.println("| q     - Back to main menu");
		break;
	    case 6:
		System.out.println("+--====== Test Menu ======--");
		System.out.println("| f  - Print all forms and functions "
				   + "used by the current language");
		System.out.println("|");
		System.out.println("| q  - Back to main menu");		
		break;
	    }
	    System.out.println("+---=======("+LANGUAGES[language]+":"+GAMES[game]+")==========---");
	    System.out.print("> ");
	}
    }
    public static void main(String args[]) {
	// Find user arguments
	int port = DEFAULT_PORT;
	boolean err = false, interactive = true;
	String prgname = "HitServerFrontEnd";
	if (args.length < 1) err = true;
	else prgname = args[0];
	for (int i = 1; i < args.length && !err; i++) {
	    if (args[i].equals("--port") && i < args.length-1) {
		port = Integer.parseInt(args[++i]);
	    } else if (args[i].equals("-ni")) interactive = false;
	    else  err = true;
	}
	if (err) {
	    System.out.println("Usage: " + prgname + " [--help] [--port <num>] [-ni]");
	    System.exit(0);
	}
	// Try 
	HitServerFrontEnd hsfe = new HitServerFrontEnd(port);
	try {		
	    // Try and see if the server is up and running
	    hsfe.hc.connect();
	    hsfe.hc.close();
	    System.out.println("Server is running!");
	    if (interactive) hsfe.run();
	} catch (IOException x) {
	    System.out.println("Server is NOT running!");
	}
    }
}

