package visl.rasmus.hitserver.server;

import java.awt.Button;
import java.awt.Event;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Panel;
import java.awt.TextArea;
import java.io.IOException;
import java.util.Enumeration;

import visl.rasmus.hitserver.client.HitXtraClient;
import visl.rasmus.hitserver.gui.DiagnoseServer;
import visl.rasmus.hitserver.gui.ManageLanguages;

/** This class is a frontend to the HitServer class 
 * @author Rasmus Lock Larsen, VISL March 2001 */
public class HitServerFrontEndGUI extends Frame implements HitServerInterface {
    int port;
    boolean err;

    public HitServerFrontEndGUI(int port, boolean err) {
	this.port = port;
	this.err = err;
    }
    public void run() {
	Panel p = new Panel();
	GridBagLayout gbl = new GridBagLayout();
	p.setLayout(gbl);
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 1;
	c.weighty = 1;
	c.gridwidth = GridBagConstraints.RELATIVE;
	TextArea ta = new TextArea(4,50);
	ta.setEditable(false);
	ta.setText("Manage the languages.\nThat is change the games supported\n"+
		   "by the languages change the highscores, change the paths to\n"+
		   "the sentences and the trees.");
	gbl.setConstraints(ta, c);
	p.add(ta);
	c.gridwidth = GridBagConstraints.REMAINDER;
	Button b = new Button("Manage Languages");
	if (err) {
	    b.disable();
	    ta.disable();
	}
	gbl.setConstraints(b, c);
	p.add(b);

	ta = new TextArea(4, 50);
	ta.setEditable(false);
	ta.setText("Manage the HitServer.\n"+
		   "Diagnose the server, make sure it is up and running.\n" +
		   "Restart the server.");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(ta, c);
	p.add(ta);
	b = new Button("Manage the HitServer");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(b, c);
	p.add(b);
	
	add("Center", p);

	p = new Panel();
	p.add(new Button("Quit"));
	add("South", p);

	pack();
	show();
    }
    public boolean handleEvent(Event e) {
	if (e.id == Event.WINDOW_DESTROY) 
	    System.exit(0);
	if (e.target instanceof Button &&
	    e.id == Event.ACTION_EVENT) {
	    Button b = (Button)e.target;
	    if (b.getLabel().equals("Manage Languages")) {
		Frame frame = new ManageLanguages(port, this);
		frame.pack();
		frame.resize(700,500);
		this.hide();
		frame.show();
	    } else if (b.getLabel().equals("Manage the HitServer")) {
		Frame frame = new DiagnoseServer(port, this, err);
		frame.pack();
		frame.resize(700, 500);
		this.hide();
		frame.show();
	    } else if (b.getLabel().equals("Quit")) {
		postEvent(new Event(this, Event.WINDOW_DESTROY, null));
		return true;
	    }
		
	}
	return false;
    }
    public static void main(String args[]) {
	// Find user arguments
	int port = DEFAULT_PORT;
	boolean err = false, interactive = false;
	String prgname = "HitServerFrontEndGUI";
	if (args.length < 1) err = true;
	else prgname = args[0];
	for (int i = 1; i < args.length && !err; i++) {
	    if (args[i].equals("--port") && i < args.length-1) {
		port = Integer.parseInt(args[++i]);
	    } else if (args[i].equals("-i")) interactive = true;
	    else  err = true;
	}
	if (err) {
	    System.out.println("Usage: " + prgname + " [--help] [--port <num>] [-i]");
	    System.exit(0);
	}
	HitXtraClient hc = new HitXtraClient("localhost", port, null, null);
	try {		
	    // Try and see if the server is up and running
	    hc.connect();
	    hc.close();
	    Enumeration en = hc.sendQuery(GET_VERSION).elements();
	    String version = null;
	    if (en.hasMoreElements()) version = (String)en.nextElement();
	    System.out.println("Server is running!");
	    System.out.println("Version: " + version);
	} catch (IOException x) {
	    System.out.println("Server is NOT running!");
	    err = true;
	}
	if (interactive) (new HitServerFrontEndGUI(port, err)).run();
    }
}

