package visl.rasmus.resources;

import java.applet.Applet;
import java.applet.AudioClip;
import java.io.DataInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;

/**
* ResourceBundle-like class for encapsulation of the comments of a given 
* language.
* @author Rasmus Lock Larsen, VISL, Nov. 2000
*/
public class Sounds {
    public static final int 
	POSITIVE = 0,
	NEGATIVE = 1,
	HELP = 2,
	GOOD = 3,
	FAIR = 4,
	POOR = 5,
	GOTHELP = 6,
	WELCOME = 7,
	DOUBLEWORK = 8;
    protected static boolean debug = false;
    static Applet applet = null;
    static String soundbase = null;
    static String soundURL = null;
    /*
     * The {type, sound} pairs as array.
     */
    static String[][] soundarray = null;
    /*
     * The {type, sound} pairs as Hashtable.
     */
    static Hashtable sounds = null;
	

    /**
     * Read comments from url.
     *@param soundUrl url to read sounds from.
     *@exception MalformedURLException if any of the URL
     */
    public static void localize(String soundUrl) throws 
    MalformedURLException, IOException{
	soundURL = soundUrl;
	sounds = null;
    }
	
	
    /**
     * Read a name=value set from an url
     * Format of the file pointed to: ({num}'='{String}'\n')*
     */
    static String[][] readSet(String url) throws 
    MalformedURLException, IOException{
	return readSet(new URL(url));
    }
	
    /**
     * Read a name=value set from an url
     * Format of the file pointed to: ({num}'='{String}'\n')*
     */
    static String[][] readSet(URL url) throws MalformedURLException, 
    IOException {
	Vector set = new Vector();
	String s = null;
	DataInputStream d = null;
	d = new DataInputStream(url.openStream());
	s = d.readLine(); // read the first line
	s.trim();
	String abbr,full; // abbreviated and full name of symbol
	int equ; // position of the '=' character
	while(s != null){ // while not EOF
	    equ = s.indexOf('=');
	    if (s.startsWith("//") ||
		s.startsWith("#")) ; // Comments
	    else if(equ>=0){
		abbr = s.substring(0,equ);
		full = s.substring(equ+1,s.length());
		set.addElement(new String[]{abbr,full}); 
		// Add the (abbr,full name) pair to the set of symbols
	    }
	    s = d.readLine(); // read the next line.
	    if (s != null) s.trim();
	}
		
	int n = set.size(); // number of symbols
	String[][] symbols = new String[n][];
	for(int i=0; i<n; i++){
	    symbols[i] = (String[]) set.elementAt(i);
	}
	return symbols;
    }
	
    static void initSound(int i) {
	if (soundarray != null) {
	    if (sounds == null) sounds = new Hashtable();
	    Integer key = null;		
	    try {
		key = Integer.valueOf(soundarray[i][0]);
	    } catch (Exception e) {
		if (debug) System.out.println("Error reading sound (" +
					      soundarray[i][1] + "): Extracting key");
	    }
	    try {
		if (key != null) {
		    if (sounds.containsKey(key)) {
			((Vector)sounds.get(key)).
			    addElement(applet.
				       getAudioClip(new URL(soundbase),
						    soundarray[i][1]));
			if (debug) System.out.print("{"+key+","+soundarray[i][1]+"}");
			if (debug) System.out.print(".");
		    } else {
			Vector v = new Vector();
			v.addElement(applet.
				     getAudioClip(new URL(soundbase),
						  soundarray[i][1]));
			if (debug) System.out.print("["+key+","+soundarray[i][1]+"]");
			sounds.put(key, v);
			if (debug) System.out.print(".");
		    }
		}
	    } catch (Exception e) {
		if (debug) System.out.println("Malformed sound-URL: " + soundbase +
					      soundarray[i][1]);
		//e.printStackTrace();
	    }
	}
    }
    /**
     * Initialize tables. Called by the localize method.
     *@param soundset Array of {"type", "sound"} pairs.
     */
    static void initTables(String[][] soundset) {
	if (soundset != null && applet != null) {
	    soundarray = soundset;
	    sounds = new Hashtable();
	    if (debug) System.out.print("Loading sounds");
	    // Get all the sounds
	    for (int i = 0; i < soundarray.length; i++) {
		initSound(i);
	    }
	    if (debug) System.out.println("done!");
	    //new SoundLoader(applet, soundbase, sounds, soundarray);
	}
    }	
    
    public static AudioClip getSound(int t) {
	Integer type = new Integer(t);
	if (sounds == null) loadSounds();
	if (sounds == null) return null;
	Vector v = null;
	if (sounds.containsKey(type)) 
	    v = (Vector)sounds.get(type);
	if (v == null) {
	    if (debug) System.out.println("No Sounds of type " + type.intValue());
	    return null;
	}	
	// The first sound is used in 75% of the cases...
	return (AudioClip)v.elementAt((int)(Math.random()*v.size()));
    }
    public static void playSound(int t) {
	AudioClip ac = getSound(t);
	if (ac != null) ac.play();
    }
    public static void setApplet(Applet aplt, String sndbase) {
	applet = aplt;
	soundbase = sndbase;
	// Clear the database
	sounds = null;
	soundarray = null;
    }
    public static void loadNames() {
	try {
	    if (debug) System.out.println("soundURL = " + soundURL);
	    soundarray = readSet(soundURL);
	} catch (Exception e) {
	    if (debug) System.out.println("Exception while loading filenames:"+
					  e);
	}
    }
    public static int getSoundCount() {
	if (soundarray != null) {
	    return soundarray.length;
	}
	return 0;
    }
    public static void loadSound(int i) {
	if (soundarray != null) {
	    initSound(i);
	}
    }
    static void loadSounds() {
	try {
	    if (soundarray == null) 
		soundarray = readSet(soundURL);
	    initTables(soundarray);
	} catch (Exception e) {
	    if (debug) System.out.println("Exception while loading sounds...");
	}
    }
    public static void setDebugging(boolean b) { debug = b; }
}



