package visl.wordfall.gui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

import visl.wordfall.resources.Colors;
import visl.wordfall.resources.Miscellaneous;
import visl.resources.ShortToLong;
import visl.resources.DebugFrame;

public class FormBox extends Canvas {
    static protected FontMetrics fm = null;

    protected String form;
    protected int w, h, index;
    protected Dimension dim, dimlock;
    protected boolean active, blinking;

    public FormBox(String form, int index) {
	this.index = index;
	this.form = form;
	if (fm == null) 
	    fm = getFontMetrics((Font)Miscellaneous.get("font.wordfall.formbox"));
	setFont((Font)Miscellaneous.get("font.wordfall.formbox"));
	//DebugFrame.get().addInfo("FormBix has index "+index+" and form " + form +" = "+ ShortToLong.getTrans(form));
	Color color = Colors.get(ShortToLong.getTrans(form));
	if (color == null){
	    /****HACK***********
		 Under windows some arabic symbols in UTF-8 encoding are not 
		 read correctly. If the order of the forms are changed this 
		 might have to be changed too.*/
	    switch (index){
	    case 17://cl-vs
		color = new Color(219, 105, 105);
		break;
	    case 5://n-ela
	    case 4://n-int
		color =  new Color(0,0,255);
		break;
	    default:
		break;
	    }
	}
	setForeground(color);
	setBackground(Colors.get("formbox"));
	w = fm.stringWidth(form);
	h = fm.getHeight();
	dim = null;
	dimlock = null;
	blinking = false;
    }
    public void paint(Graphics g) {
	Dimension d = size();
	if (d != null) {
	    // Draw 
	    g.setColor(getForeground());
	    g.drawString(form, (d.width-w)/2, (d.height+h/2)/2);
	} else System.out.println("no dim!");
    }
    public Dimension preferredSize() {
	if (fm == null) {
	    fm = getFontMetrics((Font)Miscellaneous.get("font.wordfall.formbox"));
	    setFont((Font)Miscellaneous.get("font.wordfall.formbox"));
	    w = fm.stringWidth(form);
	    h = fm.getHeight();	
	}
	if (dim == null) dim = new Dimension(2*w+8, h+4);
	if (dimlock != null && dimlock.width >= dim.width) return dimlock;
	return dim;
    }
    public void resize(Dimension d) {
	super.resize(d);
	dimlock = d;
	setBackground(Colors.get("formbox.highlight"));
	repaint();
	//super.invalidate();
    }
    public void reset() {
	dimlock = null; 
	setBackground(Colors.get("formbox"));
	repaint();
	//super.invalidate();
    }
    public String getForm() { return form; }
    private void doSleep(int msec) {
	try {
	    Thread.sleep(msec);
	} catch (InterruptedException x) {;}
    }
    public void doBlink() {
	Color bgcol = getBackground();
	setBackground(Colors.get("formbox.error"));
	update(getGraphics());
	doSleep(200);
	setBackground(Colors.get("formbox"));
	update(getGraphics());
	doSleep(200);
	setBackground(Colors.get("formbox.error"));
	update(getGraphics());
	doSleep(200);
	setBackground(Colors.get("formbox"));
	update(getGraphics());
	doSleep(200);
	setBackground(bgcol);
	update(getGraphics());
    }
    public int getIndex() { return index; }
}
